#!/bin/bash

########## README ###########
# This script launches gen as an array job on the Lyon cluster.
# The script assumes that this copy of km3 has been obtained via a simple "cvs co km3" command in the user's home directory. If this is the case, then setting:
# USER=[user] will correctly set pointers. e.g. for me:
USER=cjames
KM3_V=v5r1
# If km3 has been obtained in any other way, then the following assignation will likely need to be modified:
KM3DIR=/sps/km3net/users/${USER}/${KM3_V}
# It is also possible that for a personalised version of km3, the following variables will need to be modified:
PLATFORM=Linux
GENEXE=$KM3DIR/${PLATFORM}/gen${KM3_V}.exe

# The local medium (water) must be specified via a 'MEDFILE', which describes scattering and absorption of light. For km3net, the standard is:
MEDFILE=${KM3DIR}/dat/gen-water-optical-partic-nemo-0.0075.dat

# In order to specify the output (e.g. in case of multiple gen runs), use a TAG:
# TAG='my_tag_'
TAG='orca_200m_'
# The output directories for the five runs default to:
# ${KM3DIR}/scripts/${TAG}muon_100gev
# ${KM3DIR}/scripts/${TAG}elec_100gev
# ${KM3DIR}/scripts/${TAG}elec_10gev
# ${KM3DIR}/scripts/${TAG}elec_1gev
# ${KM3DIR}/scripts/${TAG}elec_100mev
# Gen will output one '.desc' file for each run, and [n_rad_bins] '.gen' files containing photon statistics. Each of the latter will be about 1 GB in size.

# The radial files used differ between runs, and are specific to each particle-type. The product [number of initial particles] x [fraction of photons written] is limited by the ~1 GB output size per file in f77. The exact combination of rad file and number of particles has been optimised to limit variation from too few initial events while keeping processing time to a minimum. These combinations can be toggled via altering the RADFILE and NEVENTS variables.


if [ ! -d ${KM3DIR} ]
then
	echo "Directory $KM3DIR does not exist - have you specified the correct user (= ${USER} ) and directory ( = ${KM3DIR} )?"
	exit
fi


if [ ! -f "$GENEXE" ]
then
   echo "Error: gen executable ${GENEXE} not readable"
   exit
fi

# set Lyon submission options, scripts, and log directory
LDIR=$KM3DIR/scripts/km3_log
if [ ! -d ${LDIR} ]
then
	mkdir -vp $LDIR
fi

SCRIPT=${KM3DIR}/scripts/submitted-orca-gen.sh
if [ ! -f "${SCRIPT}" ]
then
   echo "Error: gen script ${SCRIPT} does not exists"
   exit
fi

GRID_OPTIONS="-P P_antares -V -l ct=30:00:00 -l vmem=2.0G -l fsize=30G -l sps=1 -l xrootd=1 -l oracle=1 -o $LDIR -e $LDIR"
# add this to the gen options is you are using software level 6
# -l os=sl6

########## 100 gev muons [~90 min] #########

FILEROOT=muon-100gev_${KM3_V}
ENERGY=100
PTYP=5
NEVENTS=6500
RADFILE=${KM3DIR}/dat/gen-write_mu_0.1.dat
OUTPUTDIR=$KM3DIR/scripts/${TAG}muon_100gev

echo 'launching command: '
echo qsub $GRID_OPTIONS $SCRIPT $FILEROOT $NEVENTS $PTYP $ENERGY $GENEXE $RADFILE $OUTPUTDIR $MEDFILE
qsub $GRID_OPTIONS $SCRIPT $FILEROOT $NEVENTS $PTYP $ENERGY $GENEXE $RADFILE $OUTPUTDIR $MEDFILE

########## 100 gev electrons [~15 hr] #########

FILEROOT=elec-100gev_${KM3_V}
ENERGY=100
PTYP=3
NEVENTS=130
RADFILE=${KM3DIR}/dat/gen-write_0.01.dat
OUTPUTDIR=$KM3DIR/scripts/${TAG}elec_100gev

echo 'launching command: '
echo qsub $GRID_OPTIONS $SCRIPT $FILEROOT $NEVENTS $PTYP $ENERGY $GENEXE $RADFILE $OUTPUTDIR $MEDFILE
qsub $GRID_OPTIONS $SCRIPT $FILEROOT $NEVENTS $PTYP $ENERGY $GENEXE $RADFILE $OUTPUTDIR $MEDFILE

########## 10 gev electrons [~15 hr] #########

FILEROOT=elec-10gev_${KM3_V}
ENERGY=10
PTYP=3
NEVENTS=1300
RADFILE=${KM3DIR}/dat/gen-write_0.01.dat
OUTPUTDIR=$KM3DIR/scripts/${TAG}elec_10gev

echo 'launching command: '
echo qsub $GRID_OPTIONS $SCRIPT $FILEROOT $NEVENTS $PTYP $ENERGY $GENEXE $RADFILE $OUTPUTDIR $MEDFILE
qsub $GRID_OPTIONS $SCRIPT $FILEROOT $NEVENTS $PTYP $ENERGY $GENEXE $RADFILE $OUTPUTDIR $MEDFILE

########## 1 gev electrons [~90 mins] #########

FILEROOT=elec-1gev_${KM3_V}
ENERGY=1
PTYP=3
NEVENTS=1300
RADFILE=${KM3DIR}/dat/gen-write_0.1.dat
OUTPUTDIR=$KM3DIR/scripts/${TAG}elec_1gev

echo 'launching command: '
echo qsub $GRID_OPTIONS $SCRIPT $FILEROOT $NEVENTS $PTYP $ENERGY $GENEXE $RADFILE $OUTPUTDIR $MEDFILE
qsub $GRID_OPTIONS $SCRIPT $FILEROOT $NEVENTS $PTYP $ENERGY $GENEXE $RADFILE $OUTPUTDIR $MEDFILE

########## 100 mev electrons [~90 mins] #########

FILEROOT=elec-100mev_${KM3_V}
ENERGY=0.1
PTYP=3
NEVENTS=13000
RADFILE=${KM3DIR}/dat/gen-write_0.1.dat
OUTPUTDIR=$KM3DIR/scripts/${TAG}elec_100mev

echo 'launching command: '
echo qsub $GRID_OPTIONS $SCRIPT $FILEROOT $NEVENTS $PTYP $ENERGY $GENEXE $RADFILE $OUTPUTDIR $MEDFILE
qsub $GRID_OPTIONS $SCRIPT $FILEROOT $NEVENTS $PTYP $ENERGY $GENEXE $RADFILE $OUTPUTDIR $MEDFILE
