C*/
      SUBROUTINE muon_scattered_hits( part_num, seg_num, seg_e_loss,
     1           om_num, rho, cos_band, theta_om, phi_om)

*     Created:  9-OCT-97
*     Contact:  M. Moorhead, OXFORD.
*     Verified:

*     Purpose and Method:-
*     ====================
c     Calculate the number of scatter hits for a given OM and 
c     A given muon segment. First, determine if we are in the low or high
c     bands (cos_band below or above h_cos_ch).
c     Then find the average poisson 
c     probability of a hit by sampling the mu_sca_prob array by linear 
c     interpolation in the rho, cos_band, theta_om and phi_om dimensions. 
c     The hit time of each hit (if any)
c     is then found by sampling the mu_sca_time array (in 5 dimensions ). 
c     The OM hits (if any) are then placed into the hit structure record.

*     In/Out Variables:-
*     ==========
c     in        part_num        number of particle to which segment belongs
c     in	seg_num   	number of segment in seg array
c     in        seg_e_loss      energy loss in this segment
c     in        rho             magnitude of rho_vec
c     in        cos_band        cos(angle between seg_dir and rho_vec) 
c     in        theta_om        angle between om_dir and rho_vec
 
*     Common Block Access:-
*     ===================
c     /km3/  	in 		holds structures for KM3

*     Assumptions:-
*     ===========
c     the `cone' of direct light is defined with repsect to the segment center
c     and direction of muon at segment center.
 
*     Revision History:-
*     ================
c     V1	M. Moorhead	Created routine

      INCLUDE 'km3.inc'
      INCLUDE 'km3-interpol.inc'

      INTEGER*4  num_hits, part_num, seg_num, ran_poisson, bin(5,2)
      INTEGER*4  idum, i_hit, i, j, k, l, m, om_num, i_b
      REAL*4     rho, theta_om, point(5), weight(5,2)
      REAL*4     avg_num_hits, cos_band, phi_om
      REAL*4     ran1, hit_time, seg_e_loss
*
      integer    itbin
      real       vnhits(40000), vthits(40000), vxhits(40000), tempo
      real       time_min, this_time
      integer*4  ipol, i_p, locatr
      real       muon_inter_time, r1,r2
*
      ipol = INTPOL  ! Time interpolation method

*
c first find out if cos_band is below or above the cherenkov angle

      if ( cos_band .lt. h_cos_ch ) then
c we are in the low bands
          i_b = 1
          point(2) = (h_cos_ch - cos_band)**mu_sca_power(i_b,2)
      else 
c we are in the high bands
          i_b = 2
          point(2) = (cos_band - h_cos_ch)**mu_sca_power(i_b,2)
      endif

c now set the values of the point in question in the other dimensions,
c which must be adjusted to the correct powers
      
* Used to not compute hits - now limits radius
      IF (rho.lt.2.1) THEN
        point(1) = 2.1**mu_sca_power(i_b,1)
      ELSE
      	point(1) = rho**mu_sca_power(i_b,1)
      ENDIF
      point(3) = theta_om
      point(4) = phi_om
      

c now we find the bin values which straddle the current point where we are
c in the mu_dir_prob array.

      do i = 1,4    ! loop over four geometrical dimensions
         if ( point(i).le. mu_sca_min(i_b,i) ) then
c we use the first two bins
            bin(i,1) = 1 
            bin(i,2) = 2
         else if ( point(i) .ge. mu_sca_max(i_b,i) ) then
c we use the last two bins
            bin(i,1) = mu_sca_nbins(i_b,i) - 1 
            bin(i,2) = mu_sca_nbins(i_b,i)
         else
c we are inside the linearly spaced array
            bin(i,1) = 1 + int( (point(i) - mu_sca_min(i_b,i) ) / 
     1                 mu_sca_step(i_b,i) )
            bin(i,2) = bin(i,1) + 1
         endif
      enddo

c now we find the weights for each of these bins, depending on how
c far they are from the point in question. This is the linear 
c interpolation

      do i = 1, 4  ! loop over 4 geometrical dimensions
         weight(i,2) = point(i) - ( mu_sca_min(i_b,i) + 
     1                 ( bin(i,1) - 1 ) * mu_sca_step(i_b,i) ) 
         weight(i,1) = mu_sca_min(i_b,i) + ( bin(i,2) - 1 ) * 
     1                 mu_sca_step(i_b,i) - point(i) 
      enddo 

c now we do the weighted sum of all the relevant bins in the mu_sca_prob
c array which surround the point. There are a 2**n points to consider
c where n=4 is the number of geometrical dimensions

      avg_num_hits = 0.0

      do i = 1, 2  ! loop over straddling bins in radial distance
         do j = 1,2 ! loop over straddl. bins in the cosine track direction
            do k = 1,2 ! loop over straddl. bins in the OM theta direction
               do l = 1,2 ! loop over straddl. bins in the OM phi direction
                  avg_num_hits = avg_num_hits + 
     1            mu_sca_prob(i_b,bin(1,i),bin(2,j),bin(3,k),bin(4,l)) 
     2            * weight(1,i) * weight(2,j) * weight(3,k) * 
     3            weight(4,l)  
               enddo 
            enddo
         enddo
      enddo

c normalise the weighted average correctly

      avg_num_hits = avg_num_hits / mu_sca_step_prod(i_b)

c remember that mu_sca_prob is the log(prob) so we take an exponential
c to get the prob itself

      avg_num_hits = exp(avg_num_hits)

c now we adjust for small fluctuations in average Nhits according to the
c energy loss in this particular segment and we scale to average e_loss
c in the segments used to generate the HIT distributions
c This step also includes normalisation for the relative track lengths.
c If the 'hit' track length is twice that of km3, then the average eloss
c will be twice that in km3, and the ratio seg_e_loss/h_ave... will
c typically be ~0.5. When km3 groups tracks into sets of 3,9, or 27
c however, this scaling changes. Because this scaling step is dangerous (it
c discounts photons from the muon itself) the following scaling should
c be modified. To correct it, some portion of the avg_num_hits
c should be unscaled, being scattered hits from the muon itself.
c Note that the average eloss scales with track length, so this is safe.

      avg_num_hits = avg_num_hits * seg_e_loss /
     1               h_ave_e_loss_hit
c This is what would be needed for a 'proper' treatment
c   avg_num_hits = avg_num_hits*elec_frac*seg_e_loss/h_ave_e_loss_hit
c + avg_num_hits*muon_frac*current_seg_length_ratio

c now get the poisson distributed number of hits by sampling a poisson
c distribution

      num_hits = ran_poisson( avg_num_hits )

      if (num_hits.gt.5000) num_hits = 5000

      if ( num_hits .lt. 1) then
c there are no hits and we return
         return
      endif

*
* ----------------------------------------------------------------------
*
* Define the time window = 4000.  ns  \__ time_max = time_min + 4000.*10.
* Define the time bin    =    0.1 ns  /
*
      if (num_hits .eq. 1) then
        vnhits(1) = 0.
        vthits(1) = 0.
      else
        do i=1,40000
          vnhits(i) = 0.
          vthits(i) = 0.
        enddo
      endif
* sometimes the hit_time is negative -> safety window of 10ns ... 
*        time_min = om_time - 10. 
      time_min = p_t0(part_num) - 10. 

* -------------------------------------------------------------------------
* Compute inverse cumulative function values
* for time interpolation scattered photons
      if (ipol.ne.0) then 
        call muon_inter_ini( rho, cos_band, theta_om, phi_om )
      endif

c now loop over hits, determining each hit time by sampling the mu_sca_time
c distributions

c Implements first-order timing correction: assuming t ~ r
      r1=(bin(1,1)-1)*mu_sca_step(i_b,1)+mu_sca_min(i_b,1)
      r2=(bin(1,2)-1)*mu_sca_step(i_b,1)+mu_sca_min(i_b,1)
      r1=r1**(1./mu_sca_power(i_b,1))
      r2=r2**(1./mu_sca_power(i_b,1))
      weight(1,1)=weight(1,1)*rho/r1
      weight(1,2)=weight(1,2)*rho/r2

c prevents cos-extrapolation, which can lead to craziness.
      if ( point(2).le. mu_sca_min(i_b,2) ) then
            point(2) = mu_sca_min(i_b,2)
	    weight(2,2) = point(2) - ( mu_sca_min(i_b,2) + 
     1                 ( bin(2,1) - 1 ) * mu_sca_step(i_b,2) ) 
            weight(2,1) = mu_sca_min(i_b,2) + ( bin(2,2) - 1 ) * 
     1                 mu_sca_step(i_b,2) - point(2)
	    
         else if ( point(2) .ge. mu_sca_max(i_b,2) ) then
	    point(2) = mu_sca_max(i_b,2)
	    weight(2,2) = point(2) - ( mu_sca_min(i_b,2) + 
     1                 ( bin(2,1) - 1 ) * mu_sca_step(i_b,2) ) 
            weight(2,1) = mu_sca_min(i_b,2) + ( bin(2,2) - 1 ) * 
     1                 mu_sca_step(i_b,2) - point(2)
      	 endif
      

      do i_hit = 1, num_hits

c find the bins and weights in the time dimension (5) by sampling randomly
c between the mu_sca_nbins(i_b,5) time bins
      	 
	 point(5) =  ran1(idum) * mu_sca_nbins(i_b,5)
*
* ------------------------------------------------------- *
*  ipol=0  Linear      interpolation taking two  points   *
*  ipol=1  Polynomical interpolation taking thre points   *
*  ipol=2  Rational    interpolation taking thre points   *
* ------------------------------------------------------- *
*
      	 if (ipol.eq.0) then 
*  Random intervals in inverse cumulative distribution are not 
*  equally space so we need a different approach to know weights
*  and bins in linear interpolation

      	    if (H_CHEB .eq. 1) then
      	       i_p = locatr(x_cheb,mu_sca_nbins(i_b,5)+1,point(5))-1
      	    else if (H_CHEB .eq. 0) then
	       i_p = int(point(5))
	    else if (point(5) .lt. mu_sca_nbins(i_b,5)/2 ) then
	       i_p = int(mu_sca_tc*(point(5)**mu_sca_itp))
	    else
	       i_p = int(mu_sca_nbins(i_b,5) - mu_sca_tc*
     &               ((mu_sca_nbins(i_b,5)-point(5))**mu_sca_itp))
	    endif
	    
           bin(5,1) =  abs(i_p)
           bin(5,2) =  bin(5,1) + 1
      	    if (H_CHEB .eq. 1) then
	       weight(5,2) = point(5) - x_cheb(bin(5,1)) 
               weight(5,1) = x_cheb(bin(5,2)) - point(5)
	    else if (H_CHEB .eq. 0) then
	       weight(5,2) = point(5)-i_p
	       weight(5,1) = 1.-weight(5,2)
	    else
	       weight(5,2) = point(5) - mu_sca_tx(bin(5,1))
	       weight(5,1) = mu_sca_tx(bin(5,2))-point(5)
	    endif
c now we do the weighted sum of all the relevant bins in the mu_sca_time
c array which surrounds the point. There are a 5**n points to consider
c where n=5 is the number of dimensions

           hit_time = 0.0
           do i = 1, 2          ! loop over straddling bins in the first geom dimension
             do j = 1,2         ! loop over straddling bins in the 2nd  geom  dimension
               do k = 1,2       ! loop over straddl. bins in the 3rd geom. dimension
                 do l = 1,2     ! loop over straddl. bins in 4th geom. dimension
                   do m = 1,2   ! loop over straddling bins in time dimension
                     hit_time = hit_time + mu_sca_time(i_b,bin(1,i), 
     1                    bin(2,j),bin(3,k),bin(4,l),bin(5,m)) * 
     2                    weight(1,i) * weight(2,j) * weight(3,k) *
     3                    weight(4,l) * weight(5,m)  
                   enddo
                 enddo 
               enddo
             enddo
           enddo
c normalise the weighted average correctly
           hit_time = hit_time / mu_sca_step_prod(i_b)
*
           hit_time = hit_time / (weight(5,2)+weight(5,1))
         else

           hit_time = muon_inter_time(i_b,point(5),ipol)

         endif

c place the hit info in the hit buffer

         gen_nhits = gen_nhits + 1
         this_time = hit_time + seg_t_center(seg_num)
	 
         if (num_hits .eq. 1) then
           vnhits(1) = 1.0
           vthits(1) = this_time
         else
c-           itbin = (nint(this_time*10.) - int(time_min*10.) + 1)
           itbin =
     &     ((nint(this_time*10.) - int(time_min*10.))*1./(tmerg*10.))
     &     + 1

           if (itbin.gt.40000 .or. itbin.le.0) then
* Write the hit parameters immediatly if the hit time is too large
             nlines = nlines + 1
             write(50,110)
     &            nlines,
     &            g_om_id_ant_om(om_num),
     &            1.0,
     &            this_time,
     &         -1*p_part_id(part_num),
     &            p_track_id(part_num),
     &            p_iparent(part_num)
           else
             vnhits(itbin) = vnhits(itbin) + 1.0
	     vxhits(itbin) = this_time
c-             vthits(itbin) = (int(time_min*10.) + itbin - 1 )*0.1
	    vthits(itbin) = int(time_min) + real(itbin)*tmerg
     &                      - tmerg/2.

           endif
         endif
* find the earliest and latest
         if (this_time.gt.tmaxh) tmaxh = this_time
         if (this_time.lt.tminh) tminh = this_time
*
* -------------------------------------------------------
*
       enddo
*
       if (num_hits .eq. 1) then
         nlines = nlines + 1
	 if (H_REC_HIT_ORIGIN .eq. 1) then
	       write(50,111)
     &            nlines,
     &            g_om_id_ant_om(om_num),
     &            vnhits(1),
     &            vthits(1),
     &         -1*p_part_id(part_num),
     &            p_track_id(part_num),
     &            p_iparent(part_num),
     &	          seg_num
      	 else
            write(50,110)
     &             nlines,
     &             g_om_id_ant_om(om_num),
     &             vnhits(1),
     &             vthits(1),
     &          -1*p_part_id(part_num),
     &             p_track_id(part_num),
     &             p_iparent(part_num)
         endif
       else
*
         do i = 1, 40000
           if (vnhits(i) .gt. 0.) then
             nlines = nlines + 1

	     if (vnhits(i).eq.1) tempo = vxhits(i)
	     if (vnhits(i).gt.1) tempo = vthits(i)
      	     if (H_REC_HIT_ORIGIN .eq. 1) then
               write(50,111)
     &            nlines,
     &            g_om_id_ant_om(om_num),
     &            vnhits(i),
     &            tempo,
     &         -1*p_part_id(part_num),
     &            p_track_id(part_num),
     &            p_iparent(part_num),
     &	          seg_num
	     else
               write(50,110)
     &            nlines,
     &            g_om_id_ant_om(om_num),
     &            vnhits(i),
     &            tempo,
     &         -1*p_part_id(part_num),
     &            p_track_id(part_num),
     &            p_iparent(part_num)
              endif
	   endif
         enddo
       endif
*
c 110  FORMAT('hit:',1X,I7,1X,I5,1X,F6.1,1X,F10.2,3(1X,I6))
 110  FORMAT('hit:',1X,I7,1X,I7,1X,F6.1,1X,F11.2,3(1X,I6))
 111  FORMAT('hit:',1X,I7,1X,I7,1X,F6.1,1X,F11.2,4(1X,I6))
 100  FORMAT(5X,'* WARNING !! MUON_SCATT_HITS routine, gen_nhits 
     &      out of range')
*
 777   CONTINUE
*
**
*
 999  CONTINUE
      end
