C* @brief The data structures and common blocks for Gen.
C*
C* The STRUCTUREs and RECORDs of the F90 code have been converted to F77 as 
C* multi-dimensional arrays. The arrays are logically arranged into groups
C* (corresponding to the original structures):
C* 
C* - PHOTON (P)
C*   - P_POS(3) : Current photon vertex
C*   - P_DIR(3) : Current photon direction
C*   - P_LENGTH : Distance travelled so far
C*   - P_energy : Photon energy in eV
C*   - P_wlen   : Wavelength in nm
C*   - P_REG    : Current region number
C*   - P_nscat  : Number of scatterings
C* - REGION (R)
C*   - R_RAD_IN(100) : Radius  of inner bounding sphere
C*   - R_RAD_OUT(100) : Radius of outer bounding sphere
C*   - R_scale_dir(100) : Scalefactors to reduce unneeded direct hits
C*   - R_scale_sca(100) : Scalefactors to reduce unneeded scattered hits
C* - HIT_BUFFER (HIT)
C*   - HIT_TIME : Photon arrival time
C*   - HIT_WEIGHT : Weight of hit
C*   - HIT_SHELL : Shell number / OM number
C*   - HIT_POS(3) : Vertex position when hit occurredm
C*   - HIT_DIR(3) : Photon direction when hit occurred
C* - CONSTANTS (C)
C*   - C_PI : Pi!
C*   - C_MAX_STEPS : Maximum number of steps in photon propagation
C*   - C_EPS : A small number to guard against round-off errors
C*   - C_SPEED : The <B>phase</B> velocity of light in water / ice
C*   - C_LDNDL : The phase to group correction lambda * dn/dlambda
C*   - C_GROUP : The <B>group</B> velocity of light in water / ice
C*   - C_wlen_to_energy : Conversion factor for nm -> eV
C*   - C_wlen_min : The minimum Cherenkov wavelength (nm)
C*   - C_wlen_max : The maximum Cherenkov wavelength (nm)
C*   - C_energy_min : The minimum Cherenkov energy (eV)
C*   - C_energy_max : The maximum Cherenkov energy (eV)
C*   - C_alpha_fine : the fine-structure constant
C* - INPUTS (IN)
C*   - maxwlens = 50 : The maximum number of wavelength bins
C*   - IN_nwavelens : Number of wavelength bins
C*   - IN_wlen(maxwlens) : Centers of wavelength bins (nm)
C*   - IN_wnum(maxwlens) : Centers of wavenumber bins (nm-1)
C*   - IN_L_ABS(maxwlens) : Absorption lengths (m)
C*   - IN_L_SCAT(maxwlens) : Scattering lengths (m)
C*   - IN_AV_COS(maxwlens) : Average cosines for scattering
C*   - IN_rad_power : Exponent for linear radius distribution
C*   - IN_MAX_HITS : Maximum bymber of hits in distributions
C*   - IN_NREGIONS : Number of regions
C*   - IN_SEED : Current seed number
C*   - IN_HIT : Current hit number
C*
C* The user inputs for controlling the program flow and initializing the
C* water models are also included in this file and read in in the routine
C* READ_DATACARDS.
C*
C* @see gen.f for a description of the scattering model parameters
C*/

C
C--- Photon structure
C
      REAL*8    P_POS(3)	! Current photon vertex
      REAL*8    P_DIR(3)	! Current photon direction
      REAL*8    P_DIR_TRUE(3)	! True current photon direction
      REAL*8    P_LENGTH	! distance traveled so far (m)
      REAL*8    P_energy	! energy (eV)
      REAL*8    P_wlen		! wavelength (nm)
      REAL*4    P_scaling       ! scaling for muon direct hits
      REAL*8    P_abs_dist    ! absorption distance for this photon
      REAL*8    P_logwgt     ! log weight of photon
      REAL*4	T_scaling	! time-scaling for muon direct hits
      REAL*4    gpindex        ! group index of photon (cwj mod 20/11/2011)
      INTEGER*4 P_REG		! Current region number
      integer*4 P_nscat		! Number of scatters that have occurred
      INTEGER*4 P_wlbin       ! wavelength bin for photon
      INTEGER*4 MAXRBINS    ! maximum number of radial bins
      COMMON /P/P_POS,P_DIR,P_DIR_TRUE,P_LENGTH,P_energy,
     *          P_wlen,P_scaling,P_REG,P_nscat,T_scaling,gpindex,
     *	        P_wlbin, P_abs_dist, P_logwgt
      PARAMETER(MAXRBINS=100)
C
C--- Region structure
C
      REAL*8    R_RAD_IN(MAXRBINS)		! Radius of inner bounding sphere
      REAL*8    R_RAD_OUT(MAXRBINS)		! Radius of outer bounding sphere
      real*4    R_scale_dir(MAXRBINS)	! Scalefactors to reduce unneeded hits
      real*4    R_scale_sca(MAXRBINS)	! for direct and scattered light

      COMMON /R/R_RAD_IN,R_RAD_OUT,R_scale_dir,R_scale_sca

C
C--- Hit buffer structure
C
      REAL*8    HIT_TIME		! Arrival time of photon
      REAL*8    HIT_WEIGHT		! weight of hit
      INTEGER*4 HIT_SHELL		! shell # = Om #
      REAL*4    HIT_POS(3)		! photon vertex when hit occurs
      REAL*4    HIT_DIR(3)		! photon direction when hit occurs

      COMMON /HIT/HIT_TIME,HIT_WEIGHT,HIT_SHELL,HIT_POS,HIT_DIR

C
C--- Constants structure
C
      REAL*8    C_PI 
      REAL*8    C_EPS               !  small distance for round-off
      REAL*8    C_SPEED             !  speed of light in water/ice (m/ns)
      REAL*8    C_LDNDL             !  phase to group correction
      REAL*8    C_GROUP             !  group velocity of light in water
      REAL*8    C_wlen_to_energy    !  convertion factor for nm to eV 
      REAL*8    C_wlen_min          !  minimum and maximum wavelengths (nm)
      REAL*8    C_Wlen_max          !  for Cherenkov distributions
      REAL*8    C_energy_min        !  minimum and maximum energies (eV) 
      REAL*8    C_energy_max        !  for Cherenkov distributions
      REAL*8    C_alpha_fine  	    !  fine-structure constant
      INTEGER*4 C_MAX_STEPS         !  max # of steps in photon prop.

      COMMON /C/C_PI,C_EPS,C_SPEED,C_wlen_to_energy,C_wlen_min,
     &          C_wlen_max,C_energy_min,C_energy_max,C_GROUP,C_LDNDL,
     &          C_alpha_fine, C_MAX_STEPS

C
C--- Inputs structure
C
      integer maxwlens
      parameter(maxwlens=50)

      REAL*8    IN_L_ABS(maxwlens)  !  absorption length in meters
      REAL*8    IN_L_SCAT(maxwlens) !  scattering length in meters
      REAL*8    IN_AV_COS(maxwlens) !  average cosine for scattering
      integer*4 IN_nwavelens		!  # of wavelength bins
      real*4    IN_wlen(maxwlens)   !  Centers of wavelength bins (nm)
      real*4    IN_wnum(maxwlens)   !  Centers of wavenumber bins (nm-1)
      REAL*4    IN_rad_power         !  exponent for liner radius distribn.
      INTEGER*4 IN_MAX_HITS          !  number of hits in distributions
      INTEGER*4 IN_NREGIONS          !  number of regions
      INTEGER*4 IN_SEED              !  current seed number
      INTEGER*4 IN_HIT               !  current hit  number
      REAL*4    IN_e_thresh   	    !  (GeV) maximum e_loss for valid muon segmnt
      
cwj arrays for wavelength speed-ups
      REAL*4 ph_indicies(maxwlens)	         ! phase velocities of wlens
      REAL*4 group_vs(maxwlens)        ! group velocities of wlens
      REAL*4 mean_ph_indicies(maxwlens)   ! phase velocity of mean w-l in bins
      REAL*4 mean_group_vs(maxwlens)   ! group velocity of mean w-l in bins
      REAL*4 max_ph_index	      	       ! max phase index over the band
      
      COMMON /WLP/ph_indicies,group_vs,mean_ph_indicies,
     & mean_group_vs,max_ph_index
      
      COMMON /IN/IN_L_ABS,IN_L_SCAT,IN_AV_COS,
     &           IN_nwavelens,IN_wlen,IN_wnum,
     &           IN_rad_power,IN_MAX_HITS,IN_NREGIONS,IN_SEED,IN_HIT,
     &	         IN_e_thresh

      INTEGER ieventnum
      REAL theta_incident, cos_theta, sin_theta
      REAL part_plab_used, photoncount, scalefactor
      REAL e_dep, e_dep_tot
      
      COMMON /gen_shower/ ieventnum, 
     &   theta_incident, cos_theta, sin_theta, part_plab_used,
     &   photoncount, scalefactor,
     &   e_dep,e_dep_tot

* Sergio 12/07/99
*
* Henyey-Greenstein function and other water properties
*
      REAL         DENS_MEDIUM,N_PHASE
      REAL         ALPHA,COSME(2),ETA0
      REAL         TAB_ETA(maxwlens)
      CHARACTER*6  WATMOD
      COMMON/KEYCOM/TAB_ETA,DENS_MEDIUM,N_PHASE,ALPHA,COSME,ETA0,
     & WATMOD

C D. Bailey 04/01/01
C   Added the user inputs

      CHARACTER*120 OUTFILE,LOGFILE,WLENFILE,RADFILE
      INTEGER NTRIGGERS,I_PART_TYPE,I_MEDIUM,I_TRACK_LEN_MANUAL
      INTEGER I_PHOT_PROP,IPHOT_WRITE,I_STEPS_WRITE,I_MUON_CKS
      REAL PART_PLAB,TRACK_LENGTH,Z_START

      CHARACTER*6 VWATMOD(2)
      INTEGER*4 ISEED

cwj  29/11/2011 added the user inputs RVEL and DEPC and CYLINDER
      INTEGER*4 REAL_DIR_V
      INTEGER*4 EM_SCAT_ID
      INTEGER*4 WL_SCAT_METH
      INTEGER*4 CYLINDER
      REAL*4   MIN_LOGWGT
      
      COMMON/USERIN/OUTFILE,LOGFILE,WLENFILE,
     &              PART_PLAB,TRACK_LENGTH,Z_START, 
     &              VWATMOD,ISEED,
     &              RADFILE,NTRIGGERS,
     &              I_PART_TYPE,I_MEDIUM,
     &              I_TRACK_LEN_MANUAL,I_PHOT_PROP,I_PHOT_WRITE,
     &              I_STEPS_WRITE,I_MUON_CKS,EM_SCAT_ID,
     &		    REAL_DIR_V, WL_SCAT_METH, CYLINDER,
     &	             MIN_LOGWGT


c  new stuff to replace old hbook methods
      INTEGER LUN_RMIN
      INTEGER OUTID_BASE
      PARAMETER(LUN_RMIN=25)
      PARAMETER(OUTID_BASE=30)
      
      INTEGER*4 counts_dir(MAXRBINS),counts_sca(MAXRBINS)  ! counts total in each shell
      INTEGER*4 counts_edir(MAXRBINS),counts_esca(MAXRBINS)  ! counts total in each shell
      INTEGER*4 counts_writ, counts_ewrit, events_writ	        ! counts total written this event
      INTEGER*4 counts_tot(MAXRBINS),counts_etot(MAXRBINS)
      INTEGER*4 counts_wdir(MAXRBINS),counts_wsca(MAXRBINS)
      INTEGER*4 counts_ewdir(MAXRBINS),counts_ewsca(MAXRBINS)
      CHARACTER OUTFILES(MAXRBINS)*120
      INTEGER*4 OUTIDS(MAXRBINS)
      CHARACTER*120 ELOSSFILE
      COMMON/COUNTS/counts_dir,counts_sca,counts_edir,counts_esca,
     & counts_writ,counts_ewrit,events_writ, counts_tot,counts_etot,
     & counts_wdir,counts_wsca,counts_ewdir,counts_ewsca,
     & OUTFILES, OUTIDS
     
c information passed from gen to hit
      
     
     
     
     

