#!/bin/bash

########## README ###########
# This script launches hit as an array job on the Lyon cluster. 'gen' must previously have been run to provide it with suitable inout, i.e. via ./run-gen-batchlyon.sh
# The script assumes that this copy of km3 has been obtained via a simple "cvs co km3" command in the user's home directory. If this is the case, then setting:
# USER=[user] will correctly set pointers. e.g. for me:
USER=cjames
KM3_V=v5r1
# If km3 has been obtained in any other way, then the following assignation will likely need to be modified:
KM3DIR=/sps/km3net/users/${USER}/$KM3_V
# It is also possible that for a personalised version of km3, the following variables will need to be modified:
PLATFORM=Linux

# In order to specify the detector, a particular 'pmt file' must be chosen. For ANTARES, this is:
# sets the PMT file to that for KM3NeT/ORCA:
PMTFILE=${KM3DIR}/dat/hit-km3net_pmt_july13.dat

# In case of multiple runs of hit (e.g. for different PMTs), you may want to 'tag' each. The default way to do this is to specify a tag via:
HITTAG='orca_200m_'
# tag for directories that the gen files lie in
GENTAG='orca_200m_'
# and to direct the output to a particular directory:
OPDIR=${KM3DIR}/scripts/${HITTAG}hit_tables

# From here, the script first checks that everything is in order (i.e. it can find all the specified files and directories), and then runs hit for each of the five standard gen outputs (1 muon table, 4 electron tables). The gen outputs are assumed to be in the following directories:
# ${KM3DIR}/scripts/${GENTAG}muon_100gev
# ${KM3DIR}/scripts/${GENTAG}elec_100gev
# ${KM3DIR}/scripts/${GENTAG}elec_10gev
# ${KM3DIR}/scripts/${GENTAG}elec_1gev
# ${KM3DIR}/scripts/${GENTAG}elec_100mev

# If this is not the case, search for the variable 'BASEDIR' in the following and modify it accordingly. Similarly, the gen 'desc' files are assumed to be named:
# muon-100gev_${KM3_V}.desc
# elec-100gev_${KM3_V}.desc
# elec-10gev_${KM3_V}.desc
# elec-1gev_${KM3_V}.desc
# elec-100mev_${KM3_V}.desc
# Again, if run-gen-batchlyon was modified or not used to generate the output, then these names will need to be modified - look for 'FILEROOT' below.

# any questions can be emailed Clancy James at:
# clancy.james@physik.uni-erlangen.de

######### setting up file structure, and checking everything is OK #######

if [ ! -d ${KM3DIR} ]
then
	echo "Directory $KM3DIR does not exist - have you specified the correct user (= ${USER} ) and directory ( = ${KM3DIR} )?"
	exit
fi

if [ ! -f "$PMTFILE" ]
then
   echo "Error: pmtfile ${PMTFILE} not readable"
   exit
fi

# hit executable
HITEXE=${KM3DIR}/${PLATFORM}/hit${KM3_V}.exe
if [ ! -f "$HITEXE" ]
then
   echo "Error: hit executable ${HITEXE} not readable"
   exit
fi

# setting up script to submit, and associated batch commands
SCRIPT_DIR=${KM3DIR}/scripts
SCRIPT=${SCRIPT_DIR}/submitted-hit.sh
if [ ! -f "${SCRIPT}" ]
then
   echo "Error: hit script ${SCRIPT} does not exists"
   exit
fi

# directory for logging
LDIR=${SCRIPT_DIR}/km3_log
if [ ! -d ${LDIR} ]
then
	echo "mkdir -vp ${LDIR}"
	mkdir -vp $LDIR
fi

# create directory for output
if [ ! -d ${OPDIR} ]
then
	echo "mkdir -vp ${OPDIR}"
	mkdir -vp $OPDIR
fi

GRID_OPTIONS="-P P_antares -V -l ct=30:00:00 -l vmem=4.0G -l fsize=30G -l sps=1 -l xrootd=1 -l oracle=1 -l os=sl6 -o $LDIR -e $LDIR"

########## 100 gev electrons #########
FILEROOT=elec-100gev_${KM3_V}
BASEDIR=${SCRIPT_DIR}/${GENTAG}elec_100gev

GENDESC=$FILEROOT.desc
if [ ! -f "$BASEDIR/$GENDESC" ]
then
   echo "Error: descfile $BASEDIR/${GENDESC} not readable"
else
	echo 'launching command: '
	echo qsub $GRID_OPTIONS $SCRIPT $BASEDIR $FILEROOT $PMTFILE $HITEXE $OPDIR $HITTAG
	qsub $GRID_OPTIONS $SCRIPT $BASEDIR $FILEROOT $PMTFILE $HITEXE $OPDIR $HITTAG
fi

########## 10 gev electrons #########
FILEROOT=elec-10gev_${KM3_V}
BASEDIR=${SCRIPT_DIR}/${GENTAG}elec_10gev

GENDESC=$FILEROOT.desc
if [ ! -f "$BASEDIR/$GENDESC" ]
then
   echo "Error: descfile $BASEDIR/${GENDESC} not readable"
else
	echo 'launching command: '
	echo qsub $GRID_OPTIONS $SCRIPT $BASEDIR $FILEROOT $PMTFILE $HITEXE $OPDIR $HITTAG
	qsub $GRID_OPTIONS $SCRIPT $BASEDIR $FILEROOT $PMTFILE $HITEXE $OPDIR $HITTAG
fi

########## 1 gev electrons #########
FILEROOT=elec-1gev_${KM3_V}
BASEDIR=${SCRIPT_DIR}/${GENTAG}elec_1gev

GENDESC=$FILEROOT.desc
if [ ! -f "$BASEDIR/$GENDESC" ]
then
   echo "Error: descfile $BASEDIR/${GENDESC} not readable"
else
	echo 'launching command: '
	echo qsub $GRID_OPTIONS $SCRIPT $BASEDIR $FILEROOT $PMTFILE $HITEXE $OPDIR $HITTAG
	qsub $GRID_OPTIONS $SCRIPT $BASEDIR $FILEROOT $PMTFILE $HITEXE $OPDIR $HITTAG
fi

########## 100 mev electrons #########
FILEROOT=elec-100mev_${KM3_V}
BASEDIR=${SCRIPT_DIR}/${GENTAG}elec_100mev

GENDESC=$FILEROOT.desc
if [ ! -f "$BASEDIR/$GENDESC" ]
then
   echo "Error: descfile $BASEDIR/${GENDESC} not readable"
else
	echo 'launching command: '
	echo qsub $GRID_OPTIONS $SCRIPT $BASEDIR $FILEROOT $PMTFILE $HITEXE $OPDIR $HITTAG
	qsub $GRID_OPTIONS $SCRIPT $BASEDIR $FILEROOT $PMTFILE $HITEXE $OPDIR $HITTAG
fi

########## 100 gev muons #########
FILEROOT=muon-100gev_${KM3_V}
BASEDIR=${SCRIPT_DIR}/${GENTAG}muon_100gev

GENDESC=$FILEROOT.desc
if [ ! -f "$BASEDIR/$GENDESC" ]
then
   echo "Error: descfile $BASEDIR/${GENDESC} not readable"
else
	echo 'launching command: '
	echo qsub $GRID_OPTIONS $SCRIPT $BASEDIR $FILEROOT $PMTFILE $HITEXE $OPDIR $HITTAG
	qsub $GRID_OPTIONS $SCRIPT $BASEDIR $FILEROOT $PMTFILE $HITEXE $OPDIR $HITTAG
fi
