#!/bin/bash

# A script to submit to the Lyon sl6 qsub system using the script "submit-km3-batchlyon.sh"

# Required input and output files

# safe function to export the name of a file, wild characters friendly (needed with non standard file names). Written by Colas Riviere
myexport () {
  if [ -z "$2" ]; then
    exitonerror "FATAL ERROR: \"myexport $1 $2\" requires 2 arguments"
  fi
  file_nb=$( ls $2 | wc -l )
  if [ ! "$file_nb" -eq 1 ]; then
    if [ "$file_nb" -eq 0 ]; then
      exitonerror "FATAL ERROR: myexport: no match for file $2"
    else
      echo "FATAL ERROR: myexport: ambiguous second argument: $2"
      echo "Possible files:"
      ls -l $2
      exitonerror "FATAL ERROR: myexport: ambiguous second argument: $2"
    fi
  fi
  file_tmp=$( ls $2 )
  export $1=$file_tmp
  env | grep $1
}

# safe function to copy a file - complains when it is not there
mycp () {
  if [ $# -ne 2 ]; then
    exitonerror "FATAL ERROR: \"mycp $1 $2\" requires 2 arguments"
  fi
  if [ ! -f "$1" ]; then
    exitonerror "FATAL ERROR: mycp: can not read file $1"
  fi
  cp -vf $1 $2
}

exitonerror () {
  ERROREXIT=TRUE
  echo "EXIT ON ERROR: $1"
  exit
}

### function to initialise km3 ###
km3_initialise () {
  echo
  echo ">>>>>>>>>>>>>>>>>>>>>>>>>>>>>>> initialising km3"
  
  # gets the km3 binaries
  mycp $KM3EXE ./km3mc.exe
  
  if [ ! -d "$TABLEDIR" ]; then
     exitonerror "Directory $TABLEDIR not found"
  fi
  
  # gets electron tables
  mycp -vf ${TABLEDIR}/km3*el*100mev*.hit .
  mycp -vf ${TABLEDIR}/km3*el*1gev*.hit .
  mycp -vf ${TABLEDIR}/km3*el*10gev*.hit .
  mycp -vf ${TABLEDIR}/km3*el*100gev*.hit .
  
  # gets electron desc files
  mycp -vf ${TABLEDIR}/km3*el*100mev*.desc .
  mycp -vf ${TABLEDIR}/km3*el*1gev*.desc .
  mycp -vf ${TABLEDIR}/km3*el*10gev*.desc .
  mycp -vf ${TABLEDIR}/km3*el*100gev*.desc .
  
  # gets muons files
  mycp -vf ${TABLEDIR}/km3*mu*.hit .
  mycp -vf ${TABLEDIR}/km3*mu*.desc .
  
  # exports the names so that km3 can find them
  myexport hbele1 "km3*el*100mev*.hit"
  myexport hbele2 "km3*el*1gev*.hit"
  myexport hbele3 "km3*el*10gev*.hit"
  myexport hbele4 "km3*el*100gev*.hit"

  myexport deele1 "km3*el*100mev*.desc"
  myexport deele2 "km3*el*1gev*.desc"
  myexport deele3 "km3*el*10gev*.desc"
  myexport deele4 "km3*el*100gev*.desc"
  
  myexport hbmuon "km3*mu*.hit"
  myexport demuon "km3*mu*.desc"

  # (probably out-dated) dispersion file
  mycp ${DATDIR}/km3-dispersion.hbook .
  myexport hbdisp "km3-dispersion.hbook"
  
  # inputs for music
  mycp ${DATDIR}/km3-music-cross-sections.dat .
  mycp ${DATDIR}/km3-music-double-diff.dat .
  mycp ${DATDIR}/km3-music-eloss.dat .
  
  # exports music input files
  myexport Meloss "km3-music-eloss.dat"
  myexport Mddxsections "km3-music-double-diff.dat"
  myexport Mxsections "km3-music-cross-sections.dat"
  
  # copies detector and event files
  mycp ${DETFILE} detector.det
  mycp ${EVTFILE} event.evt
  
  # exports these names, and defines the output file
  myexport GEOFIL detector.det
  myexport EVIFIL event.evt
  export EVOFIL=km3_event.evt
  echo "<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<< km3_initialised"
}

# actually runs km3
km3_run () {
  echo
  echo ">>>>>>>>>>>>>>>>>>>>>>>>>>>>>>> running km3"
  SEED=`date +%s`
  echo "SEED is $SEED"
  
# WARNING: Do not indent EOF !!!
./km3mc.exe << EOF
IRAND 3 ${SEED} 0 0
#  ** treat all events
INEVT 9999999
#  ** merging 1 nsec
XMERG 0.1
#  ** empty events NOT written
IEMPT 0
END
EOF

  rm -v fort.?? 
  echo "<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<< km3 run completed"
}


############################# MAIN #############################

# get inputted information
ERROREXIT=FALSE

# we now get the passed arguments

KM3EXE=$1
TABLEDIR=$2
DATDIR=$3
EVTFILE=$4
DETFILE=$5
OUTFILE=$6
echo $1 $2 $3 $4 $5 $6

# temporary name for outfile
EVTOUT="outfile.evt"

# Soft version definitions
CWJDIR=/sps/km3net/users/cjames

# init km3 - gets relevant files
km3_initialise

####### running km3 itself ########
echo
echo "============================> KM3 <============================"
time km3_run
echo "km3 simulation finished "
echo "cp $EVTOUT $OUTFILE"
mycp $EVTOUT $OUTFILE


############################################# FINISHED ###############################################

finish
