C* GEANT-required routine called at the end of each tracking step
C* Here photons are generated according to the Frank-Tamm formula for Cerenkov
C* light, and propagation begins
C* INWVOL is different from 0 when the track has reached a volume boundary  
C* ISTOP is different from 0 if the track has stopped    
C*
C* In general generated secondaries are putin the track stack unless the
C* particle is the primary muon and it is more than 100 cm from the origin
C* or the secondaries are more than 20m from the origin.
C*
C* The number of Cherenkov photons is calculated from a Poisson distribution
C* with mean:
C* \f$ \overline{N_{ch}} = 763 \times Q^{2} \times d \times sin^{2}(\theta_{ch}) \f$
C*
C* Where:
C* - Q is the particle charge
C* - d is the length of the current tracking step in cm
C*
C* There are several assumption inherent in this routine which determine
C* the physics of Cherenkov light generation which may be accurate at high
C* energies (for which the KM3 code was originally written) but may need
C* checking again later if it is to be used for the neutralino / oscillation
C* analysis.
C*
C* - The number of Cherenkov photons is taken from the parameterisation above
C*   at all energies above threshold
C* - The threshold itself is determined by the phase velocity
C* - The Cherenkov photons are generated at the Cherenkov angle determined 
C*   (correctly) by the phase velocity. At energies close to threshold, this
C*   will no longer be strictly true.
C*
C* @warning UNVERIFIED
C*/
	subroutine gustep 

      include 'gen-geant.inc'
      include 'gen.inc'
      include 'gen-hit-ntuple.inc'

      integer ran_poisson           ! returns poisson integer from avg 
      integer*4 tmp_nphotons        ! temporary store the number of photons
       real part_dir(3),phot_dir(3)	! Particle and photon directions
      real phot_dir_true(3)         ! Real photon direction
      real phot_pos(3)		! Photon position
      real ran1                   ! randon variable
      real*4 pindex               ! refractive index
      real*4 dnpe                 ! exact number of photons
      real*4 beta                 ! relative speed of the particle
      real*4 ctheta, stheta2        ! wavelength dependent Cerenkov angle
      real*4 ctheta_ch, stheta_ch     ! fixed Cerenkov angle
      real*4 wmin, wmax             ! bounds of the wavelength interval
      real*4 tgpindex	      !temporary group index variable
C	CALL GPCXYZ		! debug print of current step
 
c this stops the muon at length 100*track_length cm beyond the start point     	
      if((ipart.eq.5.or.ipart.eq.6).and.istak.eq.0.and.
     &	    abs(vect(3)).gt.100.0*(track_length+z_start)) then
C--- i.e. Do nothing
      	 go to 1
      endif
      vsum=vect(1)**2 + vect(2)**2 + vect(3)**2

C --- we are outside the generation 'sphere'
      if(vsum.gt.4000000.) then
      	 go to 1		!Do nothing
      endif

C--- If we passed all these tests, then allow generation of secondaries (keep particle)
      call gsking(0)
C
C--- Also if we pass the above tests, we record the diagnostic track segment 
C--- ntuple
C
      
      xtup3(1)=float(ieventnum)
      xtup3(2)=float(ipart)	! Particle type
      xtup3(3)=float(istak)	! GEANT variables showing where this segment
      xtup3(4)=float(ivert)	!     is in the "tree" of secondaries
      xtup3(5)=amass		! Mass (GeV)
      xtup3(6)=charge		! Charge (e)
      xtup3(7)=vect(1)/100	! Location X,Y,Z (m)
      xtup3(8)=vect(2)/100
      xtup3(9)=vect(3)/100
      xtup3(10)=vect(4)	! Direction vector Vx,Vy,Vz
      xtup3(11)=vect(5)
      xtup3(12)=vect(6)
      xtup3(13)=getot		! Total energy (GeV)
      xtup3(14)=step/100	! Length of this tracking step (m)
      xtup3(15)=destep	! Energy loss of this step (GeV)
      xtup3(16)=tofg*1.e9	! time relative to beginning of event (nsec)
      xtup3(17)=0.		! Number of CK photons (0 unless charged and fast)
				!    (see below)
C
C--- Increment deposited energy
C
      e_dep=e_dep+destep
C
C--- Calculate <Nphotons> emitted in this track segment (300 to 600 nm 
C--- wavelength) (if charge=0, no light; if beta<1/n, no light)
C--- First, see if this is a muon primary; if so, and I_MUON_CKS is off, do not
C--- create CK photons
C
      
      if(((ipart.ne.5.and.ipart.ne.6).or.istak.ne.0.or.i_muon_cks.eq.1)
	1	.and.charge.ne.0.0 .and. getot.gt.amass ) then
      	 gamma=getot/amass
      	 beta=sqrt(1.0-(1.0/gamma**2))
      	 
      	 tmp_nphotons = 0
C
C--- We may need the "standard" Cerenkov angle later for muons
C--- For electrons we do not care as long as it is a valid value

C--- if we scale to a common theta_C, calculate those parameters
      	 IF (REAL_DIR_V .EQ. 0) THEN
C determine if the entire wavelength band can emit using the central frequency only
            ctheta_ch = 1.0/(beta*n_phase)
      	    if (ctheta_ch .gt. 1.0 .and. 
     *         (ipart.eq.5 .or. ipart.eq.6)) go to 1
      	    stheta_ch = sqrt( max(0, 1 - ctheta_ch**2) )
      	 ELSE
C    use the max refractive index to determine if emission is possible
	    if (beta*max_ph_index .lt. 1.0) go to 1
	 ENDIF
	 
C--- Loop on all wavelength bins
      	 do bin_wlen = 1, in_nwavelens-1
	 
      	    wmin = in_wlen(bin_wlen)
      	    wmax = in_wlen(bin_wlen+1)
	    
C--- Check if photons can be created in this wavelength bin
C--- cos(theta) is now (approximately) the mean value over the
C--- current bin. '0' indicates that phase index is wanted.
	    ctheta = 1.0/(beta * mean_ph_indicies(bin_wlen))
      	    if (ctheta .gt. 1.0) then
      	       cycle
      	    endif
      	    stheta2 = 1. - ctheta**2
C--- Get the number of photons in this bin (Frank Tamm formula)

      	    dnpe = 2*real(c_pi)*c_alpha_fine*stheta2*step/100
     * 	          *charge**2 * (1./wmin-1./wmax) * 1.e9

C--- Throw the Poisson dice to get actual number of photons emitted
      	    nphotons = ran_poisson(dnpe)
C
C--- Increment number of photons generated in this event
C
      	    photoncount = photoncount + float(nphotons)
      	    tmp_nphotons = tmp_nphotons + nphotons
C
C--- Photon initial position is taken to be the endpoint of the track segment;
C--- this is OK, since segments are of the order of 1 cm long.  Also record
C--- particle direction here for later use in generating directions of CK 
C--- photons
C
      	    do i=1,3
      	       part_dir(i)=vect(i+3)
      	       phot_pos(i)=vect(i)/100. ! Go from cm to m
      	    end do
      	    if(i_phot_write.eq.1) then
      	       do j=1,nphotons
C
C--- Wavelength sampled uniformly in energy from c.energy_min 
C--- (1239.84/600nm=2.07 eV) to c_energy_max (1239.84/300nm=4.14eV). 
C--- This gives a 1/lambda^2 cherenkov number distribution
C
      	          P_energy = c_wlen_to_energy 
     *                  * ( 1./dble(wmin) + dble(ran1(in_seed)) *
     *                        (1./dble(wmax) - 1./dble(wmin)) )

		  P_wlen   = c_wlen_to_energy / P_energy

C--- Discard this photon if its wavelength does not allow it to exist
      	          call water_index(real(P_wlen),pindex,0)
      	          ctheta = 1.0/(beta*pindex)
      	          if (ctheta .gt. 1.0) then
      	             photoncount = photoncount - 1
      	             tmp_nphotons = tmp_nphotons - 1
      	             cycle
      	          endif

C---  gets the correct group index regardless of ref index
      	          call water_index(real(P_wlen),tgpindex,1)
      	          gpindex = tgpindex
C
C--- For each photon, give it the correct Cerenkov theta and random phi
C--- relative to the direction of this track segment
C--- In an ideal world, one would use the wavelength dependent Cerenkov 
C--- angle.  However, for muons this would require many changes in KM3, 
C--- so as a trick we use the "standard" angle and we will correct the 
C--- shell radius from (sinus standard Cer angle) / (sin local Cer angle)
C
CWJ - I have removed the 'standard' Cerenkov angle in this version
C
C--- Fill variables used by PHOTON routines
C--- Position and direction: only if not original (anti)muons
      	          if ((ipart.ne.5 .and. ipart.ne.6)
     & .or. (ipart.eq.5 .and. i_part_type.ne.5)
     & .or. (ipart.eq.6 .and. i_part_type.ne.6)) then
C   Treat the particles exactly (usualy means electrons).
      	             P_scaling = 1.0
		     P_nscat = EM_SCAT_ID
C--- Using the correct ctheta value for that wavelength
                     call scatvec(part_dir, ctheta, phot_dir)
                     nelec = nelec+1

                     do k=1,3
                     	P_pos(k)      = phot_pos(k)
                     	P_dir(k)      = phot_dir(k)
                     	P_dir_true(k) = phot_dir(k)
                     enddo
      	          else
C     Determines whether to force direct photons to be at the mean
C     Cherenkov angle or not. If so, then the 'true' photon direction
C     is also calculated - otherwise the one used is true.
      	             P_nscat=0
		     IF (REAL_DIR_V .eq. 0) THEN
C     we compensate direct photons using p_scaling
      	             	stheta2 = 1 - ctheta**2
      	             	P_scaling = stheta_ch / sqrt(stheta2)
		     	call scatvec2(part_dir, ctheta_ch, ctheta,
     *                        phot_dir, phot_dir_true)
     		     	do k=1,3
		     	   P_dir_true(k) = phot_dir_true(k)
		     	enddo
		     ELSE
C   - we are using the true angle, with no scattering needed
		     	P_scaling=1.0
		     	call scatvec(part_dir, ctheta, phot_dir)
                     	do k=1,3
		     	   P_dir_true(k) = phot_dir(k)
		     	enddo
		     ENDIF
      	             IF (ipart .ne. 5 .and. ipart .ne. 6) THEN
      	             	nelec = nelec+1
      	             ELSE
      	             	nmuon = nmuon+1
		     ENDIF

                     do k=1,3
                     	P_pos(k) = phot_pos(k)
                     	P_dir(k) = phot_dir(k)
                     enddo
                  endif
C--- Region number initialisation
      	          P_reg=1
      	          dist_to_orig=sqrt(phot_pos(1)**2+phot_pos(2)**2
	1			+phot_pos(3)**2)
      	          do k=1,in_nregions
      	             if(dist_to_orig.gt.r_rad_in(k)*P_scaling .and.
     *               	dist_to_orig.le.r_rad_out(k)*P_scaling)
     *               	P_reg=k
      	          end do
C--- Total pathlength traveled by photon
      	          P_length=0.0d0
C--- Current log weight of particle (wgt=1)
		  P_logwgt=0.0d0

C--- Fill ntuple of initial photon information
      	          xtup(1)=tofg*1.0e9-track_length/2/c_speed
      	          do k=1,3
      	             xtup(1+k)=phot_pos(k)
      	             xtup(4+k)=phot_dir(k)
      	          end do
      	          xtup(8) = P_wlen       ! Wavelength
		  xtup(9) = 1.0
C
C--- Number of scatters
C--- Counts versus shell radius histograms
C--- Treat photons emitted from electrons/positrons off muons as scattered hits
C--- unless EM_SCAT_ID is specifically set to 0
C
      	          if ( (i_part_type .eq. 5 .and. ipart .eq. 5)
     *              .or. (i_part_type .eq. 6 .and. ipart .eq. 6))
     *            then
C--- We scale all direct hits to a single bin
		     if (P_nscat .ne. 0) then
		     	write(*,*) 'P_nscat wrong, not 0!'
		     	write(*,*) i_part_type, EM_SCAT_ID, REAL_DIR_V
		     endif
      	             P_nscat=0
                  else
	             if (P_nscat .ne. EM_SCAT_ID) then
		     	write(*,*) 'P_nscat wrong, not EMSID!', P_nscat
		     	write(*,*) i_part_type, EM_SCAT_ID, REAL_DIR_V
		     endif
      	             P_nscat=EM_SCAT_ID
      	          endif

C  Calculate the integer wavelength bin for this photon
      	          CALL find_wlen_bin(P_wlen)

C--- Start propagation
      	          if(i_phot_prop.eq.1) then
      	             call propagate_photon(iflag)
      	          end if
      	       end do
      	    end if
      	 end do
C
C--- Record number of photons from this segment in diagnostic ntuple
C
      	 xtup3(17)=float(tmp_nphotons)
      end if
C
C--- Enter data in diagnostic ntuple if appropriate
C
      if(i_steps_write.ne.0) call hfn(nid_tracks,xtup3)
1     continue

      return
      end
