C* Original author:  D. Lowder<BR>
C* Original date:    2 August 1996<BR>
C*
C* This program simulates the generation of Cerenkov light by a particle in
C* ice or water, including light from any secondary particles.  It then tracks 
C* the Cerenkov photons through space with wavelength-dependent absorption and
C* scattering taken into account, recording position, direction, and arrival
C* time of photons at spherical shells of various radii centered on the origin.
C* It is intended that the data generated by this program be used by further
C* codes to create the OM hit distributions needed by the full KM3 Monte Carlo
C* code. 
C*
C* The original particle can be any GEANT particle, with any energy.  If
C* it is a muon (GEANT particle ID = 5), the program will stop tracking it
C* after 1 meter of travel.  The particle always starts on the  axis,
C* with an offset from the origin specified by the user, and travels along
C* the Z axis in the positive direction.  The user may specify how many
C* simulations are to be run for this single particle type and energy.
C*
C* This main routine initializes GEANT and PAW, then starts the
C* GEANT-based simulation process. It then extracts the information from the
C* GEANT ntuples are writes the description file.
C*
C* The water models (as specified by WATMOD) are 
C* parameterised according to "Light and Water. Radiative Transfer in Natural 
C* Waters", C.D.Mobley, Academic Press, ISBN 0-12-502750-8, 1994
C*
C* 1)  MEDSEA : Simulation of a linear combination of Henyey-Greenstein phase
C* functions (MOBLEY p.106):
C* 
C* \f$ \beta^{\prime} = \alpha \times \beta (g1,\theta) + (1-\alpha) \times \beta (g2,\theta) \f$
C*
C*  COSME(1) = first  possible value of <cos> = g1<BR>
C*  COSME(2) = second possible value of <cos> = g2<BR>
C*  ALPHA    = gives the probability of having COSME(1) or COSME(2)<BR>
C* 
C* 2/  PARTIC 
C*  Linear combination of Rayleigh and Particles scattering (MOBLEY p.111)
C*
C*  \f$ \beta_{global} = \eta \times \beta_{Rayleigh} + (1-\eta) \times \beta_{Particles} \f$
C*
C*  Eta gives the probability of having Rayleigh or Particles
C*
C* @see gen.inc for a description of the data structure
C*
C* @warning UNVERIFIED
C*/

 	program gen

	include 'gen-geant.inc'
	include 'gen.inc'
	include 'gen-paw.inc'
	include 'gen-hit-ntuple.inc'
	include 'gen-water.inc'

	INTEGER*4 i, ierr
        real*4 tarray(2)
        real*4 new_counts, new_scale_dir(100), new_scale_sca(100) 
	character*120 histname
        character*2 c_opt
	character*2 intstring2
	character*1 intstring1
        logical  file_exists, od
	REAL*4 index, group
	integer genbit

! GEANT initialisation
	call timest(1.e7)	! lots of time!
	call gzebra(150000)
	call uginit

! Reset GEANT variable NEVENT to our NTRIGGERS
	nevent=ntriggers
	
! PAW initialization
	call hlimit(-pawsiz)

	CALL TIMEL(TL)
        write(6,*) 'Starting the job ... Current CPU time=',TL

*
* Initialize random number generator (RANLUX).
* Default Luxurity level (3).
* Starting point fixed by in_seed (from DATACARDS).
	CALL RLUXGO(3,in_seed,0,0)
*

! Reset PAW ERROR condition...
c	IERROR = 0
c        ieventnum = 0

				! User inputs from specified files

	open(unit=lun_nt2,file=wlenfile,status='old',
     1       form='formatted',IOSTAT=ierr)

	IF(ierr.NE.0) THEN
	  WRITE(*,*) ' Error opening wlenfile file ',ierr
	  write(*,*) ' expected file ',wlenfile
	  STOP
	END IF
	read(lun_nt2,*) in_nwavelens
	do i=1,in_nwavelens
		read(lun_nt2,*) in_wlen(i),in_l_abs(i),
     1		in_l_scat(i),in_av_cos(i)
     
	end do
*
* Read particle scattering tabulated data and build inverse cumulative function
*
	IF (WATMOD.EQ.'partic') THEN
           DO i=1, ndat_beta
              READ(lun_nt2,*) part_ang(i),part_beta(i)
           ENDDO
           CALL MOBLEY
        ENDIF

	close(lun_nt2)

*
* Calculates the maximum phase index over the wavelength band
*
      max_ph_index=1.0
      DO i=1, in_nwavelens
      	 CALL water_index(in_wlen(i),index,0)
	 CALL water_index(in_wlen(i),group,1)
	 ph_indicies(i)=index
	 group_vs(i)=group
	 if (index .gt. max_ph_index) max_ph_index=index
	 IF (i .lt. in_nwavelens) THEN
	    CALL water_index((in_wlen(i)+in_wlen(i+1))/2., index, 0)
	    CALL water_index((in_wlen(i)+in_wlen(i+1))/2., group, 1)
	    mean_ph_indicies(i) = index
	    mean_group_vs(i) = group
	    if (index .gt. max_ph_index) max_ph_index=index
	 ENDIF
      ENDDO
      IF (REAL_DIR_V .eq. 0) THEN
      	 max_ph_index=n_phase
      ENDIF
      write(*,*) 'max_ph_index = ', max_ph_index
*
* --------------------------------------------------------------
*
* Dump values of wave-length, absortion lenght, scatter lenght and av_cos
* into a kumac file. This file can be read, inside PAW, using
*       w_abs_scat_len.kumac
*
	open(unit=25,file='gen_wlen.kumac',status='unknown',
     1       form='formatted')
*
         if (in_nwavelens.le.1) goto 345
         write(25,*) ' ve/cr wlen(50)  r _ '
         write(25,400) (in_wlen(i),i=1,in_nwavelens-1)
         write(25,401) in_wlen(in_nwavelens)
         write(25,*) ' ve/cr l_abs(50)  r _ '
         write(25,400) (in_l_abs(i),i=1,in_nwavelens-1)
         write(25,401) in_l_abs(in_nwavelens)
         write(25,*) ' ve/cr l_scat(50)  r _ '
         write(25,400) (in_l_scat(i),i=1,in_nwavelens-1)
         write(25,401) in_l_scat(in_nwavelens)
         write(25,*) ' ve/cr av_cos(50)  r _ '
         write(25,400) (in_av_cos(i),i=1,in_nwavelens-1)
         write(25,401) in_av_cos(in_nwavelens)
*
 400    FORMAT(F10.5,' _ ')
 401    FORMAT(F10.5)

 345	continue
* --------------------------------------------------------------

	open(unit=lun_nt2,file=radfile,status='old',
     1       form='formatted',IOSTAT=ierr)

	IF(ierr.NE.0) THEN
	  WRITE(*,*) ' Error opening radfile file ',ierr
	  STOP
	END IF
*
	read(lun_nt2,*) sim_rad
	read(lun_nt2,*) in_nregions
	do i=1,in_nregions
      	   read(lun_nt2,*) r_rad_in(i),r_rad_out(i),r_scale_dir(i),
     &	r_scale_sca(i)
        end do
	read(lun_nt2,*) in_rad_power
	close(lun_nt2)
c  initialises written counts to be 0
	do i=1,in_nregions
	   counts_dir(i)=0
	   counts_sca(i)=0
	   counts_tot(i)=0
	   counts_wdir(i)=0
	   counts_wsca(i)=0
        end do
	counts_writ=0
	events_writ=0
	e_dep_tot=e_dep

! input constants
 	c_speed     = 0.299792458d0   ! speed of light in vacuum (m/ns)
	c_Pi        = 3.1415927d0
	c_eps       = 1.0d-4    	! small number to avoid round off probs
	c_max_steps = 10000
        c_wlen_to_energy = 1239.84d0   ! rev part physics (2012) 
        c_wlen_min    = 300.0d0            ! minimum wavelegth for photons (nm)
        c_wlen_max    = 600.0d0            ! maximum wavelegth for photons (nm)
	c_energy_min  = c_wlen_to_energy / c_wlen_max ! min photon energy  (eV)
 	c_energy_max  = c_wlen_to_energy / c_wlen_min ! max photon energy  (eV)
        c_alpha_fine = 7.297352537d-3 ! review particle physics 2012
! Open output file, but first check to see if it exists
      write(*,*) 'Opening outfiles!'
      IF (I_PHOT_PROP .ne. 0 .and. I_PHOT_WRITE .ne. 0) THEN

c searches for .gen part of name
         do i=1,120
	    if (outfile(i:i+3).eq. '.gen') then
	       genbit=i-1
	       go to 789
	    endif
	 enddo
	 write(*,*) 'end of outfile not found. Aborting...'
	 
c      	 genbit=index(outfile,'.dat')
c	 write(*,*) 'found .dat in outfile at position ',genbit
c	 write(*,*) 'prefix is thus ',outfile(:genbit)
789	 continue
	 
	 do i=1,in_nregions-1
	    outids(i) = OUTID_BASE+i
	    if (i .lt. 10) then
	       write(*,'(I1)') i
	       write(intstring1,'(I1)') i
	       outfiles(i)=outfile(:genbit)//'_'//intstring1//'.gen'
	    else
	       write(intstring2,'(I2)') i
	       outfiles(i)=outfile(:genbit)//'_'//intstring2//'.gen'
	    endif
	    
	    inquire( file = outfiles(i) , exist = file_exists )
	    
	    if ( .not. file_exists ) then
      	       OPEN(UNIT=outids(i),FILE=outfiles(i),ACCESS='DIRECT',
     & STATUS='NEW', FORM='UNFORMATTED',RECL=reclen,IOSTAT=istat)
      	    else
      	       OPEN(UNIT=outids(i),FILE=outfiles(i),ACCESS='DIRECT',
     & STATUS='OLD', FORM='UNFORMATTED',RECL=reclen,IOSTAT=istat)
      	    endif
	    
	    inquire( file = outfiles(i) , exist = file_exists,
     & opened=od )
	    write(*,*) 'outfile ',i,' is ',outfiles(i),
     & ' istat=',istat, ' id=',outids(i), ' open=',od
	 enddo
      ELSE
      	 write(*,*)' No photons being written... file not opened'
      ENDIF
      
! Make ntuples and histograms for...

! Sample of all photons is recorded here -- each shell has a different
! scalefactor (see above) used to decide what fraction of photons get
! recorded for that shell
      IF (i_phot_prop.ne.0) THEN
      	 do i=1,in_nregions-1
      	    histname(1:19)='photons from shell '
      	    write(histname(20:21),'(I2)') i
*                encode(2,'(i2.2)',histname(20:21)) i
      	 end do
      ENDIF
! Open log file (.desc) and write out some info:
	open(unit=lun_nt2,
     &       file=logfile,status='unknown',form='formatted')
	write(lun_nt2,100) outfile
100	format('  Output file name is ',a120)
	if(i_medium.eq.1) then
          write(lun_nt2,*) i_medium,' = I_MEDIUM (Medium is ice)'
	else if (i_medium .eq. 2) then
          write(lun_nt2,*) i_medium,' = I_MEDIUM (Medium is water)'
	else
	  write(lun_nt2,*) i_medium,' = I_MEDIUM (Medium is seawater)'
	end if
	write(lun_nt2,*) n_phase,' = phase index of refraction'
	write(lun_nt2,*) dens_medium,' = density'

        write(lun_nt2,203)  WATMOD

      	write(lun_nt2,204)  in_l_abs(9),in_wlen(9)

        write(lun_nt2,205)  in_l_scat(9),in_wlen(9)
        write(lun_nt2,206)  ALPHA
        write(lun_nt2,207)  COSME(1)
        write(lun_nt2,208)  COSME(2)
        write(lun_nt2,209)  ETA0

 203	format(A10,' = Water model name')
 204	format(2X,F8.3,' = absorption length at ',F8.3,' nm')
 205	format(2X,F8.3,' = scattering length at ',F8.3,' nm')
 206	format(2X,F8.3,' = alpha')
 207	format(2X,F8.3,' = cosme 1')
 208	format(2X,F8.3,' = cosme 2')
 209	format(2X,F8.3,' = eta')

        write(lun_nt2,*) EM_SCAT_ID, '   = NSCAT for direct e-'
	write(lun_nt2,*) REAL_DIR_V, '   = true v (all as scat?)'
	
	write(lun_nt2,101) radfile
101	format(' Region file = ',a120)
	write(lun_nt2,*)
     &    sim_rad,' = Max. distance to propagate photons'
	write(lun_nt2,*)
     &    in_nregions-1,' = Number of active regions'
	write(lun_nt2,*) ' Region    Outer radius    Scalefactor(dir)',
	1	'    Scalefactor(scat)'
	do i=1,in_nregions-1
		write(lun_nt2,223) i, r_rad_out(i), r_scale_dir(i),
 	1	r_scale_sca(i)
 223	format(I6,3X,3F16.10)
		write(*,*) 'gen.f ->',i, r_rad_out(i), r_scale_dir(i),
 	1	r_scale_sca(i)

	end do
	write(lun_nt2,*) in_rad_power,' rad power for linear inter. '	
	if(i_part_type.eq.5) then
		write(lun_nt2,*) i_part_type,' = I_PART_TYPE (mu+)'
	else if(i_part_type.eq.6) then
		write(lun_nt2,*) i_part_type,' = I_PART_TYPE (mu-)'
	else if(i_part_type.eq.2) then
		write(lun_nt2,*) i_part_type,' = I_PART_TYPE (e+)'
	else if(i_part_type.eq.3) then
		write(lun_nt2,*) i_part_type,' = I_PART_TYPE (e-)'
	else
		write(lun_nt2,'(I10)') i_part_type,' = I_PART_TYPE'
	end if
	write(lun_nt2,*) part_plab,' = Initial particle momentum'
	write(lun_nt2,*) 
     &        nevent,' = total # of events (electrons) or steps (muons)'
	write(lun_nt2,*) 
     &        track_length,' = Muon/shower track length'
	write(lun_nt2,*) z_start,' = Z start position (m)'
	
* Initialize functions to compute absorption length, pmt_eff ...
c	call ini_optic(0,1,55.)
* <--
      	 
! GEANT call to start events processing 
	call grun

! End of run
	call uglast

c  closes the output files
      do i=1,in_nregions-1
      	 CLOSE(outids(i))
      enddo
      
c Most of this info is only relevant to muons
      write(lun_nt2,*) IN_e_thresh, ' muon eloss thresh (GeV)'
      write(lun_nt2,*) events_writ, ' events written'
      write(lun_nt2,*) e_dep_tot/events_writ, ' mean eloss (GeV)'
      write(lun_nt2,*) counts_writ, '   total photons written'
      write(lun_nt2,*) 'SHELL DIRECT SCATTERED TOTAL FILENAME'
      do i=1,in_nregions-1
	 write(lun_nt2,353)  i, counts_wdir(i), counts_wsca(i),
     &	 counts_wdir(i)+counts_wsca(i),outfiles(i)
      end do
353   format(I3,3(1x,I8,1x),a120)

c     write further info which need not be read by hit
      write(lun_nt2,*) i_phot_prop,' = I_PHOT_PROP'
	write(lun_nt2,*) i_phot_write,' = I_PHOT_WRITE'
	write(lun_nt2,*) i_steps_write,' = I_STEPS_WRITE'
	write(lun_nt2,102) wlenfile
102	format(' Wavelength file = ',a120)
	write(lun_nt2,*) in_nwavelens,' = # of wavelength bins'
	write(lun_nt2,*) 'Wlength (nm)  Labs (m)   Lscat (m)',
     1                   '  Costheta (ave)'
c write wavelength info
	do i=1,in_nwavelens
	        write(lun_nt2,202) in_wlen(i),in_l_abs(i),
	1		in_l_scat(i),in_av_cos(i)
202		format(3(4x,f7.1),4x,f7.3)
	end do

! Extract direct and scattered counts in shells and write to .desc file
      write(lun_nt2,*) ' Finished generating events'
      write(lun_nt2,*) 'Shell number   Direct hits  Scattered hits'
      do i=1,in_nregions-1
      	 write(lun_nt2,224) i,counts_wdir(i),counts_wsca(i)

224	 format(I6,2X,2F14.3)

      end do
      close(lun_nt2)

c we loop over the shells resetting the scalefactors in each according
c to how many hits each shell had in this run [removed now]

	do i=1,in_nregions-1
	 write(*,*) 'COUNTS generated: ', i, counts_dir(i), counts_sca(i),
     &               ', and written: ', counts_wdir(i), counts_wsca(i)
        end do

	stop
	end
