C*/
      SUBROUTINE Process_band

*     Created:  12-June-96
*     Contact:  M. Moorhead, LBNL.
*     Verified:

*     Purpose:-
*     =============
*     process the hits in a given band on a given shell
*     into the N_det and t_dis arrays for probability of hit and
*     time distribution (both as a function of theta_om and phi_om).
*     The radius of the shell is determined by the h_i_r index
*     The band number is determined by the h_i_band index
*     The type of band is determined by the value of h_scat
c     0 = direct hits, 1 = scattered hits.

*     In/Out Variables:-
*     ==========
 
*     Common Block Access:-
*     ===================
c     /hit/  	in 		holds structures for HIT program

*     Assumptions:-
*     ===========
*     For each photon hit (on the band) the (theta_om x phi_om)
*     array of OM orientations is found by calling find_om_directions.
*     Then for each hit x Om orientation the pmt effective area is looked up 
*     based on the dot product of the photon direction and the OM direction,
*     and based on the wavelength of the photon.

*     Revision History:-
*     ================
c     V1	M. Moorhead	Created routine

      include 'hit.inc'

      integer*4 i, j, k, l
      real*4    t_dis(0:max_t_scat), dot, hit_pos(3)
      real*4    om_dir(3), t_ch
      real*4    local_pmt_eff_area, band_dot
      real*4    eff_area_pmt
      real*4    t_list(max_hits)
      real*4    w_list(max_hits)
      real*4    cos_th_ch, delta_theta_ch
      real*4    tmp_phi, tmp_theta, sin_th
      real*4    pindex
      real*4    ph_dir(3), norm_ph_dir, sqrt_bit

c internal structure for recording weights of each photon in (theta_om,phi_om)
c bins and also the total weights for all bins

C      structure /stom/ 
         
	 real*8 om_tot_weight
         real*4 om_weight(max_hits)
	 
C      end structure
C      record /stom/ om( max_theta_om_dir, max_phi_om_scat )
c check to make sure this band has at least 1 hit
      if ( p_nhits .eq. 0 ) then

         write (*,*) 'ERROR: band has no hits: r = ', h_i_r,
     1               ' band = ', h_i_band, ' h_scat = ', h_scat

         return

      endif

c major loop over the OM directions, looking up the effective area of the OM
c as a function of wavelength and polar angle between the OM and the photon.
      do k = 1, h_nbins_om_theta(h_scat)
         do l = 1, h_nbins_om_phi(h_scat)

            om_tot_weight = 0.0000000000

      	    call vzero(t_list,max_hits)
      	    call vzero(w_list,max_hits)

c minor loop over photons
      	    do i = 1, p_nhits 
c correct direction of the photon from angular dispersion of the
c Cerenkov angle ; relevant only for muon direct hits
	 
      	       do j = 1,3
                  ph_dir(j) = p_dir(i,j)
      	       enddo
c find the OM orientations with respect to the photon position
      	       do j = 1,3
                  hit_pos(j) = p_pos(i,j)
               enddo
      	 
               call find_om_direction( hit_pos, om_dir, k, l)
	    
c correct for the angular acceptance of the band area, which goes as
c abs( cos( theta ) ) where theta is the angle between the normal to the
c band at the hit point and the direction vector of the hit
               band_dot = 0.
               do j = 1,3
                  band_dot = band_dot + p_pos(i,j) * ph_dir(j)
               enddo
*
* Protection : abs value must be between 0 and 1 inclusive
	       band_dot=MAX(0.,MIN( 1.0,
     &             	   abs( band_dot )/r_rad(h_i_r)))

               if ( band_dot .lt. h_min_band_dot ) then 
c we clip the distribution in band_dot so that we do not get statistical
c spikes in the HIT distributions from dividing by small values of band_dot
                  band_dot = h_min_band_dot
      	       endif

c calculate the dot product of the photon REVERSE direction and the OM
c direction vector  
               dot = 0.
               do j = 1,3 ! integer for xyz directions
                  dot = dot - om_dir(j) * 
     1                  ph_dir(j)
               enddo
	       
* dot =  1.  -->  max. eff.
* dot = -1.  -->  min. eff. (zero)
*
      	       local_pmt_eff_area = eff_area_pmt( p_wlen(i), dot ) ! new function
c now we calcuate the weight for this hit with this OM direction by 
c multiplying together all the relevant factors:
c i)   pmt_eff_area = effective area of PMT
c ii)  r_prescale = prescale factor used in GENERATE-PHOTONS
c iii) b_area = area for this band
c iv)  band_dot = factor for correcting band area for this hit direction
c v)   h_good_events = number of primary particles 
c NOTE: we must correct for all 'direct' hits...
               if ( h_scat .eq. 0 ) then
c this is a direct hit and we use the prescale factor for direct hits
                  om_weight(i) = local_pmt_eff_area / 
     1 ( r_prescale(0,h_i_r) * b_area(h_scat,h_i_r,h_i_band) * 
     2 band_dot * h_good_events ) *P_wgt(i)
      	          

               else 
c this is a scattered hit and we use the prescale factor for scatterd hits
                  om_weight(i) = local_pmt_eff_area / 
     1 ( r_prescale(1,h_i_r) *  b_area(h_scat,h_i_r,h_i_band) * 
     2 band_dot * h_good_events ) *P_wgt(i)
      	          

               endif
	       
     
               om_tot_weight = om_tot_weight + 
     1                              om_weight(i)

            enddo
c now we have the total weights for the (theta_om, phi_om) bin
c and the individual weights for the hits (hit,theta_om,phi_om)
c So, we can call make_t_dis to get the time distribution for the
c (theta_om, phi_om) bin

c copies the true times to a temporary list for sorting. Why not do this earlier?...
            call ucopy( p_time, t_list, max_hits)

C D. Bailey: 22/11/00
C For the change to F77, do an explicit loop on the last index of the 
C array to perform the copy rather than call the CERNLIB routine
C ucopy( om_weight(k,l,*), w_list, max_hits)

            do i = 1,max_hits
              w_list(i) = om_weight(i)
            end do

            call make_t_dis( p_nhits, 
     1           t_list, w_list, 
     2           om_tot_weight, t_dis )

            if ( h_scat  .eq. 1 ) then
c fill the t_dis_scat output buffer

               do i = 0, h_nbins_time(h_scat)
                  t_dis_scat(h_i_r,h_i_band,k,l,i) = t_dis(i)
               enddo

c and fill the N_det_scat output buffer

               n_det_scat(h_i_r,h_i_band,k,l) = om_tot_weight

            else

c fill the t_dis_dir output buffer

               do i = 0, h_nbins_time(h_scat)
                  t_dis_dir(h_i_r,h_i_band,k,l,i) = t_dis(i) 
               enddo

c and fill the N_det_dir output buffer

               n_det_dir(h_i_r,h_i_band,k,l) = om_tot_weight

            endif
         enddo
      enddo
      end
