C*/
      SUBROUTINE elec_scattered_hits( part_num, om_num, 
     1           rho, cos_band, theta_om, phi_om)

*     Created:  26-mar-98
*     Contact:  M. Moorhead, OXFORD.
*     Verified:

*     Purpose and Method:-
*     ====================
c     Calculate the number of scattered hits for a given OM and 
c     A given electron shower. First, determine if we are in the low or high
c     bands (cos_band below or above h_cos_ch).
c     Then find the average poisson 
c     probability of a hit by sampling the el_sca_prob array by linear 
c     interpolation in the energy, rho, cos_band, theta_om and phi_om
c     dimensions. 
c     The hit time of each hit (if any)
c     is then found by sampling the el_sca_time array. 
c     The OM hits (if any) are then placed into the hit structure record.
c     
c     The interpolation assumes the following:
c          - the time-distribution is independent of the hit probability
c            in the radial and energy dimensions
c          - the time distribution is proportional to the hit probability
c            in the angular (2 OM + 1 track) dimensions
c        Hence, the four energy- and radial- points are calculated first
c           by averaging the time-distributions from the three angular
c           dimensions according to the hit probabilities, and then
c           the final distribution is calculated by weighting these
c           according to how 'close' the OM is in parameter space

*     In/Out Variables:-
*     ==========
c     in        part_num        number of particle (shower)
c     in        rho             magnitude of rho_vec
c     in        cos_band        cos(angle between seg_dir and rho_vec) 
c     in        theta_om        angle between om_dir and rho_vec
c     in        phi_om 	        azimuthal orientation of om_dir about rho_vec
 
*     Common Block Access:-
*     ===================
c     /km3/  	in 		holds structures for KM3

*     Assumptions:-
*     ===========
c     the light is defined with repsect to the shower  maximum
 
*     Revision History:-
*     ================
c     V1	M. Moorhead	Created routine

      INCLUDE 'km3.inc'
      INCLUDE 'km3-interpol.inc'

      INTEGER*4  num_hits, part_num, ran_poisson, bin(6,2)
      INTEGER*4  idum, i_hit, i, j, k, l, m, n, om_num, i_b
      REAL*4     rho, theta_om, point(6), weight(6,2)
      REAL*4     avg_num_hits, cos_band, phi_om
      REAL*4     ran1, hit_time, bit
      
*      new variables for new weighting system
      REAL*4 new_time,r1,r2
      REAL*4 wgts(2,2,2,2,2), wgt_sum(2,2)
*
      integer    itbin, ifather,type_hit
      real       vnhits(40000), vthits(40000), vxhits(40000), tempo
      real       time_min, this_time
      integer*4  ipol, i_p, locatr
      real       elec_inter_time
*
      ipol = INTPOL  ! Time interpolation method

      TYPE_HIT = -3
*
c ###########
c bins are:
c 1 energy
c 2 radius
c 3 angle wrst shower axis
c 4 theta_om
c 5 phi_om
c 6 time
c #######

c first find out if cos_band is below or above the cherenkov angle
      
      if ( cos_band .lt. h_cos_ch ) then
c we are in the low bands
          i_b = 1
          point(3) = (h_cos_ch - cos_band)**el_sca_power(i_b,3)
      else 
c we are in the high bands
          i_b = 2
          point(3) = (cos_band - h_cos_ch)**el_sca_power(i_b,3)
      endif

c now set the values of the point in question in the other dimensions,
c which must be adjusted to the correct powers
      
      point(1) = log10( p_e0(part_num) ) 
      point(2) = rho**el_sca_power(i_b,2)
      point(4) = theta_om
      point(5) = phi_om

* 10-Feb-2000
* If hits too close, then assume 2.1m distance
* 
      IF (rho.lt.2.1) THEN
        point(2) = 2.1**el_sca_power(i_b,2)
      ENDIF
      
c now we find the bin values which straddle the current point where we are
c in the mu_dir_prob array.

      do i = 1,5    ! loop over 1 energy + four geometrical dimensions
         if ( point(i).le. el_sca_min(i_b,i) ) then
c we use the first two bins
            bin(i,1) = 1 
            bin(i,2) = 2
c ensures that near-direct scattered photons can not be faster than light
c MODMODMODMOD
c	    if (i .eq. 3) then
c	       point(i) = el_sca_min(i_b,i)
c	    endif
         else if ( point(i) .ge. el_sca_max(i_b,i) ) then
c we use the last two bins
            bin(i,1) = el_sca_nbins(i_b,i) - 1 
            bin(i,2) = el_sca_nbins(i_b,i) 
         else
c we are inside the linearly spaced array 
            bin(i,1) = 1 + int( (point(i) - el_sca_min(i_b,i) ) / 
     1                 el_sca_step(i_b,i) ) 
            bin(i,2) = bin(i,1) + 1
	    
         endif
      enddo

*
c	We don't want the first sphere to be 'inside' the cascade.... why???
c         Shower energy  |  First band
c   ---------------------|-----------------------
c             E < 1 GeV  | @ 0.8 m   sphere nr.1
c     1 GeV < E < 10 GeV | @ 2   m   sphere nr.2
c    10 GeV < E          | @ 4   m   sphere nr.3
c
c check this - it appears to have a hard-coded radial bin
      if ( el_sca_nbins(1,2).eq.19 .and. el_sca_min(1,2).lt.1.0) then
        if (bin(1,1).eq.2 .and. bin(2,1).eq.1) then
          bin(2,1) = 2
          bin(2,2) = 3
        else if (bin(1,1).eq.3 .and. bin(2,1).le.2) then
          bin(2,1) = 3
          bin(2,2) = 4
        endif
      endif

c now we find the weights for each of these bins, depending on how
c far they are from the point in question. This is the linear 
c interpolation

      do i = 1, 5  ! loop over 1 energy + 4 geometrical dimensions
         weight(i,2) = point(i) - ( el_sca_min(i_b,i) + 
     1                 ( bin(i,1) - 1 ) * el_sca_step(i_b,i) ) 
         weight(i,1) = el_sca_min(i_b,i) + ( bin(i,2) - 1 ) * 
     1                 el_sca_step(i_b,i) - point(i) 
      enddo 
      
      
c now we do the weighted sum of all the relevant bins in the el_sca_prob
c array which surround the point. There are a 2**n points to consider
c where n=5 is the number of dimensions

      avg_num_hits = 0.0
      do i = 1, 2  ! loop over straddling bins in the first dimension (energy)
         do j = 1,2 ! loop over straddl. bins in the second dimension (radius)
	    wgt_sum(i,j) = 0.0
            do k = 1,2 ! loop over straddl. bins in the 3rd dimension (angle)
               do l = 1,2 ! loop over straddl. bins in the 4th dimension (theta OM)
                  do m = 1,2 ! loop over straddl. bins in 5th dimension (phi OM)

c                    this is exactly the same calculation as before - but wgts are stored
		     wgts(i,j,k,l,m)=
     1		     el_sca_prob(i_b,bin(1,i),bin(2,j),bin(3,k),
     2               bin(4,l),bin(5,m)) * weight(1,i) * 
     3               weight(2,j) * weight(3,k) * weight(4,l) *
     4               weight(5,m)
		     avg_num_hits = avg_num_hits + wgts(i,j,k,l,m)
		     
c     	        ###     For a possible new interpolation method    ###
c		     now converts this to the linear domain
c		     wgts(i,j,k,l,m)=
c     &	                exp(wgts(i,j,k,l,m)/el_sca_step_prod(i_b))
c     	             records the total weights individually for the
c     	             energy and radial dimensions
c		     wgt_sum(i,j) = wgt_sum(i,j)+wgts(i,j,k,l,m)
		     
                  enddo
               enddo 
            enddo
         enddo
      enddo
      
c normalise the weighted average correctly
      
      avg_num_hits = avg_num_hits / el_sca_step_prod(i_b)
      
c remember that el_sca_prob is the log(prob) so we take an exponential
c to get the prob itself
      avg_num_hits = exp( avg_num_hits )
      
c now get the poisson distributed number of hits by sampling a poisson
c distribution
      num_hits = ran_poisson( avg_num_hits )
      if (num_hits.gt.5000) num_hits = 5000
      
      if ( num_hits .le. 0) then
c no hits and we return
          return 
      endif

* ----------------------------------------------------------------------
*
* Define the time window = 4000.  ns  \__ time_max = time_min + 4000.*10.
* Define the time bin    =    0.1 ns  /
*
      if (num_hits .eq. 1) then
        vnhits(1) = 0.
        vthits(1) = 0.
      else
        do i=1,40000
          vnhits(i) = 0.
          vthits(i) = 0.
        enddo
* 
      endif
      ifather = p_iparent(part_num)
      time_min = p_t0(ifather) - 10.
*-------------------------------------------------------------------------

* Compute inverse cumulative function values
* for time interpolation scattered photons
      if (ipol.ne.0) then 
        call elec_inter_ini(part_num, rho, cos_band, theta_om, phi_om)
      endif

c now loop over hits (if any), determining each hit time by sampling the 
c el_sca_time distributions

c Implements first-order timing correction: assuming t ~ r
      r1=(bin(2,1)-1)*el_sca_step(i_b,2)+el_sca_min(i_b,2)
      r2=(bin(2,2)-1)*el_sca_step(i_b,2)+el_sca_min(i_b,2)
      r1=r1**(1./el_sca_power(i_b,2))
      r2=r2**(1./el_sca_power(i_b,2))
      weight(2,1)=weight(2,1)*rho/r1
      weight(2,2)=weight(2,2)*rho/r2
      
c prevents energy- and cos-extrapolation.
      do i = 1,3,+2    ! loop over energy and cos angle
         if ( point(i).le. el_sca_min(i_b,i) ) then
            point(i) = el_sca_min(i_b,i)
	    weight(i,2) = point(i) - ( el_sca_min(i_b,i) + 
     1                 ( bin(i,1) - 1 ) * el_sca_step(i_b,i) ) 
            weight(i,1) = el_sca_min(i_b,i) + ( bin(i,2) - 1 ) * 
     1                 el_sca_step(i_b,i) - point(i)
	    
         else if ( point(i) .ge. el_sca_max(i_b,i) ) then
	    point(i) = el_sca_max(i_b,i)
	    weight(i,2) = point(i) - ( el_sca_min(i_b,i) + 
     1                 ( bin(i,1) - 1 ) * el_sca_step(i_b,i) ) 
            weight(i,1) = el_sca_min(i_b,i) + ( bin(i,2) - 1 ) * 
     1                 el_sca_step(i_b,i) - point(i)
      	 endif
      enddo
      
      do i_hit = 1, num_hits

c find the bins and weights in the time dimension (6) by sampling randomly
c between the el_sca_nbins(i_b,6) time bins

         point(6) =  ran1(idum) * el_sca_nbins(i_b,6)
*
* ------------------------------------------------------- *
*  ipol=0  Linear      interpolation taking two  points   *
*  ipol=1  Polynomical interpolation taking thre points   *
*  ipol=2  Rational    interpolation taking thre points   *
* ------------------------------------------------------- *
*
         if (ipol.eq.0) then 
*  Random intervals in inverse cumulative distribution are not 
*  equally space so we need a different approach to know weights
*  and bins in linear interpolation

           
           
	   if (H_CHEB .eq. 1) then
      	       i_p = locatr(x_cheb,el_sca_nbins(i_b,6)+1,point(6))-1
      	    else if (H_CHEB .eq. 0) then
	       i_p = int(point(6))
	    else if (point(6) .lt. 0.5) then
	       i_p = int(el_sca_tc*(point(6)**el_sca_itp))
	    else
	       i_p = int(el_sca_nbins(i_b,6) - el_sca_tc*
     &           ((el_sca_nbins(i_b,6)-point(6))**el_sca_itp))
	    endif
	    
           bin(6,1) =  abs(i_p)
           bin(6,2) =  bin(6,1) + 1
      	    if (H_CHEB .eq. 1) then
	       weight(6,2) = point(6) - x_cheb(bin(6,1))
               weight(6,1) = x_cheb(bin(6,2)) - point(6)
	    else if (H_CHEB .eq. 0) then
	       weight(6,2) = point(6)-i_p
	       weight(6,1) = 1.-weight(6,2)
	    else
	       weight(6,2) = point(6) - el_sca_tx(bin(6,1))
	       weight(6,1) = el_sca_tx(bin(6,2))-point(6)
	    endif
	   
c now we do the weighted sum of all the relevant bins in the el_sca_time
c array which surrounds the point. There are a 6**n points to consider
c where 6=5 is the number of dimensions
           hit_time = 0.0
c	   new_time = 0.0
	   do i = 1,2           ! loop over straddling bins in the first dimension (energy)
             do j = 1,2         ! loop over straddling bins in the 2nd  dimension (radius)
               do k = 1,2       ! loop over straddl.   bins in the 3rd dimension (angle)
                 do l = 1,2     ! loop over straddl.   bins in 4th dimension (thetaOM)
                   do m = 1,2   ! loop ovaer straddling bins in 5th dimension (phiOM)
                     do n = 1,2 ! loop over the time dimension 
c                      the new calculation - wgts in angle correspond to p(detection)

c		       new_time=new_time+el_sca_time(i_b,
c     1                      bin(1,i),bin(2,j),bin(3,k),bin(4,l),
c     2                      bin(5,m),bin(6,n))*weight(1,i)*
c     3	             	    weight(2,j)*
c     4			    weight(6,n)*wgts(i,j,k,l,m)/wgt_sum(i,j)
       
		       hit_time = hit_time + el_sca_time(i_b,
     1                      bin(1,i),bin(2,j),bin(3,k),bin(4,l),
     2                      bin(5,m),bin(6,n)) * weight(1,i) * 
     3                      weight(2,j) * weight(3,k) * weight(4,l)* 
     4                      weight(5,m) * weight(6,n)
                     enddo  
                   enddo
                 enddo 
               enddo
             enddo
           enddo
	   
c normalise the weighted average correctly
           hit_time = hit_time / el_sca_step_prod(i_b)
           hit_time = hit_time / (weight(6,2)+weight(6,1))
c	   new_time = new_time / el_sca_step_prod_d12(i_b)
c           new_time = new_time / (weight(6,2)+weight(6,1))
c      	    if (USENEW .eq. 1) then
c	       hit_time=new_time
c	    endif
	   
         else

           hit_time = elec_inter_time(i_b,point(6),ipol)

         endif

         gen_nhits = gen_nhits + 1
*
* -------------------------------------------------------
*
         this_time = hit_time + p_t0(part_num)
	 
         if (num_hits .eq. 1) then
           vnhits(1) = 1.0
           vthits(1) = this_time
         else
           itbin =
     &     ((nint(this_time*10.) - int(time_min*10.))*1./(tmerg*10.))
     &     + 1

           if (itbin.gt.40000 .or. itbin.le.0) then
* Write the hit parameters inmediatly if the hit time is too large
             nlines = nlines + 1
             write(50,110)
     &            nlines,
     &            g_om_id_ant_om(om_num),
     &            1.0,
     &            this_time,
     &            type_hit,
     &            p_iparent(part_num),
     &            part_num
           else
             vnhits(itbin) = vnhits(itbin) + 1.0
	     vxhits(itbin) = this_time
     	    vthits(itbin) = int(time_min) + real(itbin)*tmerg
     &                      - tmerg/2.

           endif
         endif
* find the earliest and latest
         if (this_time.gt.tmaxh) tmaxh = this_time
         if (this_time.lt.tminh) tminh = this_time
*
* -------------------------------------------------------
*
       enddo
*
       if (num_hits .eq. 1) then
         nlines = nlines + 1
         if (H_REC_HIT_ORIGIN .eq. 1) then
	       write(50,111)
     &            nlines,
     &            g_om_id_ant_om(om_num),
     &            vnhits(1),
     &            vthits(1),
     &            type_hit,
     &            p_iparent(part_num),
     &            part_num,
     &	          p_eff_seg(part_num)
	 else
	    write(50,110)
     &             nlines,
     &             g_om_id_ant_om(om_num),
     &             vnhits(1),
     &             vthits(1),
     &             type_hit,
     &             p_iparent(part_num),
     &             part_num
         endif
       else
*
         do i = 1, 40000
           if (vnhits(i) .gt. 0.) then
             nlines = nlines + 1

	     if (vnhits(i).eq.1) tempo = vxhits(i)
	     if (vnhits(i).gt.1) tempo = vthits(i)
      	     if (H_REC_HIT_ORIGIN .eq. 1) then
               write(50,111)
     &            nlines,
     &            g_om_id_ant_om(om_num),
     &            vnhits(i),
     &            tempo,
     &            type_hit,
     &            p_iparent(part_num),
     &            part_num,
     &	          p_eff_seg(part_num)
	     else
               write(50,110)
     &            nlines,
     &            g_om_id_ant_om(om_num),
     &            vnhits(i),
     &            tempo,
     &            type_hit,
     &            p_iparent(part_num),
     &            part_num
      	     endif
           endif
         enddo
       endif
*
c 110  FORMAT('hit:',1X,I7,1X,I5,1X,F6.1,1X,F10.2,3(1X,I6))
 110  FORMAT('hit:',1X,I7,1X,I7,1X,F6.1,1X,F11.2,3(1X,I6))
 111  FORMAT('hit:',1X,I7,1X,I7,1X,F6.1,1X,F11.2,4(1X,I6))
 100  FORMAT(5X,'* WARNING !! ELEC_SCATT_HITS routine, gen_nhits 
     &      out of range')
*
 777   CONTINUE
*
**
*
 999  CONTINUE
      end
