C*/
      SUBROUTINE PHOT_SCATTERED_HITS( part_num, om_num, 
     1           rho, cos_band, theta_om, phi_om )
*************************************************************************
*     Name           :  PHOT_SCATTERED_HITS                             *
*     Called by      :                                                  *
*     Author         :  Sergio Navas                                    *
*     Date           :  09-Oct-1998                                     *
*     Task           :  Process scattered photons comming from a        *
*                       LASER-flash into OM hits.                       *
*                                                                       *
*  In/Out Variables:                                                    *
*  -----------------                                                    *
*   in     part_num    number of particle (shower)                      *
*   in     om_num      number of current OM                             *
*   in     rho         magnitude of rho_vec                             *
*   in     cos_band    cos(angle between seg_dir and rho_vec)           *
*   in     theta_om    angle between om_dir and rho_vec                 *
*   in     part_num    number of particle to which segment belongs      *
*   in     ier         error code                                       *
*                                                                       *
*  Assumptions:                                                         *
*  ------------                                                         *
*     the light is defined with repsect to the shower  maximum          *
*                                                                       *
* Calculate the number of scattered hits for a given OM and             *
* a given photon LASER-flash. Find the average poisson probability      *
* of a hit by sampling the phot_sca_prob array by linear interpolation  *
* in the rho, cos_band, theta_om and phi_om dimensions.                 *
* The hit time of each hit (if any) is then found by sampling the       *
* phot_sca_time array.                                                  *
* The OM hits (if any) are then placed into the hit structure record.   *
*                                                                       *
*************************************************************************
*
      INCLUDE 'km3.inc'
      INCLUDE 'km3-interpol.inc'
*
      INTEGER*4  num_hits, part_num, ran_poisson, bin(5,2)
      INTEGER*4  idum, i_hit, i, j, k, l, m, om_num
      REAL*4     rho, theta_om, phi_om, point(5), weight(5,2)
      REAL*4     avg_num_hits, cos_band
      REAL*4     ran1, hit_time
*
      integer    itbin
      real       vnhits(40000), vthits(40000), vxhits(40000), tempo
      real       time_min, this_time
      integer*4  ipol, i_p, locatr
      real       inter_time
*
      ipol = INTPOL
*
* -- Ntuple inputs :
c      NSCAT = 1
c      ID_OM    = g_om_id_ant_om(om_num)
c      TYPE_HIT = -1
*
c now set the values of the point in question in the other dimensions,
c which must be adjusted to the correct powers

      point(1) = rho**phot_sca_power(1)
      point(2) = cos_band
      point(3) = theta_om
      point(4) = phi_om

c now we find the bin values which straddle the current point where we are
c in the mu_sca_prob array.

      do i = 1,4    ! loop over four geometrical dimensions
         if ( point(i).le. phot_sca_min(i) ) then
c we use the first two bins
            bin(i,1) = 1 
            bin(i,2) = 2
         else if ( point(i) .ge. phot_sca_max(i) ) then
c we use the last two bins
            bin(i,1) = phot_sca_nbins(i) - 1 
            bin(i,2) = phot_sca_nbins(i) 
         else
c we are inside the linearly spaced array 
            bin(i,1) = 1 + int( (point(i) - phot_sca_min(i) ) / 
     1                 phot_sca_step(i) ) 
            bin(i,2) = bin(i,1) + 1
         endif
      enddo

c now we find the weights for each of these bins, depending on how
c far they are from the point in question. This is the linear 
c interpolation

      do i = 1, 4  ! loop over four geometrical dimensions
         weight(i,2) = point(i) - ( phot_sca_min(i) + 
     1                 ( bin(i,1) - 1 ) * phot_sca_step(i) ) 
         weight(i,1) = phot_sca_min(i) + ( bin(i,2) - 1 ) * 
     1                 phot_sca_step(i) - point(i) 
      enddo 

c now we do the weighted sum of all the relevant bins in the phot_sca_prob
c array which surround the point. There are a 2**n points to consider
c where n=4 is the number of dimensions

      avg_num_hits = 0.0
      do i = 1, 2  ! loop over straddling bins in the first dimension (shell)
         do j = 1,2 ! loop over straddl. bins in the second dimension (band)
            do k = 1,2 ! loop over    bins in the 3rd dimension (theta_OM)
               do l = 1,2 ! loop over bins in the 4th dimension (phi_OM)
                  avg_num_hits = avg_num_hits + 
     1            phot_sca_prob(bin(1,i),bin(2,j),bin(3,k),bin(4,l))
     2            * weight(1,i) * weight(2,j) * weight(3,k) * 
     3            weight(4,l)
                enddo
            enddo
         enddo
      enddo
 211  format(4I3,4F7.5,1X,F8.6)

c normalise the weighted average correctly

      avg_num_hits = avg_num_hits / phot_sca_step_prod

c remember that phot_sca_prob is the log(prob) so we take an exponential
c to get the prob itself

      avg_num_hits = exp( avg_num_hits )

c now get the poisson distributed number of hits by sampling a poisson
c distribution

      num_hits = ran_poisson( avg_num_hits )

      if (num_hits.gt.5000) num_hits = 5000

      if ( num_hits .le. 0) then
c no hits and we return
          return 
      endif

*
* ----------------------------------------------------------------------
*
* Define the time window = 4000.  ns  \__ time_max = time_min + 4000.*10.
* Define the time bin    =    0.1 ns  /
*
      if (num_hits .eq. 1) then
        vnhits(1) = 0.
        vthits(1) = 0.
      else
        do i=1,40000
          vnhits(i) = 0.
          vthits(i) = 0.
        enddo
      endif
* sometimes the hit_time is negative -> safety window of 10ns ... 
      time_min = p_t0(part_num) - 10. 
* -------------------------------------------------------------------------

c now loop over hits (if any), determining each hit time by sampling the 
c phot_sca_time distributions

      do i_hit = 1, num_hits

c find the bins and weights in the time dimension (5) by sampling randomly
c between the phot_sca_nbins(i_b,5) time bins

        point(5) =  ran1(idum) * phot_sca_nbins(5)
*
* ------------------------------------------------------- *
*  ipol=0  Linear      interpolation taking two  points   *
*  ipol=1  Polynomical interpolation taking thre points   *
*  ipol=2  Rational    interpolation taking thre points   *
* ------------------------------------------------------- *
*
        if (ipol.eq.0) then
*
*  Random intervals in inverse cumulative distribution are not 
*  equally space so we need a different approach to know weights
*  and bins in linear interpolation

          i_p = locatr(x_cheb,phot_sca_nbins(5)+1,point(5))

          bin(5,1) =  abs(i_p)-1
          bin(5,2) =  bin(5,1) + 1
          weight(5,2) = point(5) - x_cheb(bin(5,1)) 
          weight(5,1) = x_cheb(bin(5,2)) - point(5)

c now we do the weighted sum of all the relevant bins in the phot_sca_time
c array which surrounds the point. There are a 5**n points to consider
c where n=5 is the number of dimensions

          hit_time = 0.0
          do i = 1, 2           ! loop over straddling  bins in the 1st geom dimension
            do j = 1,2          ! loop over straddling bins in the 2nd geom dimension
              do k = 1,2        ! loop over stradd. bins in the 3rd geom dimension
                do l = 1,2      ! loop over straddl. bins in 4th geom. dimension
                  do m = 1,2    ! loop over straddling  bins in time dimension
                    hit_time = hit_time + phot_sca_time(
     1                   bin(1,i),bin(2,j),bin(3,k),bin(4,l),bin(5,m))
     2                   *weight(1,i)*weight(2,j)*weight(3,k)
     3                   *weight(4,l)*weight(5,m)
                  enddo
                enddo 
              enddo
            enddo
          enddo
c normalise the weighted average correctly
          hit_time = hit_time / phot_sca_step_prod
*
          hit_time = hit_time / (weight(5,2)+weight(5,1))
*
        else

          hit_time = inter_time(point(5),ipol,om_num)

        endif
*>>>>>>>>>>>>>>>>>>>>>>>>

c place the hit info in the hit buffer
        gen_nhits = gen_nhits + 1
*
* -------------------------------------------------------
*
        this_time = hit_time + p_t0(part_num)
        if (num_hits .eq. 1) then
          vnhits(1) = 1.0
          vthits(1) = this_time
        else
          itbin = (nint(this_time*10.) - int(time_min*10.) + 1)
          if (itbin.gt.40000 .or. itbin.le.0) then
            write(*,*) 'WARNING !!! sca_phot itbin=',itbin,part_num,
     &           gen_dada_id
            itbin = 40000
          endif
          vnhits(itbin) = vnhits(itbin) + 1.0
	  vxhits(itbin) = this_time
          vthits(itbin) = (int(time_min*10.) + itbin - 1 )*0.1
        endif
* find the earliest and latest
        if (this_time.gt.tmaxh) tmaxh = this_time
        if (this_time.lt.tminh) tminh = this_time
*
* -------------------------------------------------------
*
      enddo
*
       if (num_hits .eq. 1) then
         nlines = nlines + 1
         write(50,110)
     &             nlines,
     &             g_om_id_ant_om(om_num),
     &             vnhits(1),
     &             vthits(1),
     &          -1*p_part_id(part_num),
     &             p_iparent(part_num),
     &             part_num
       else
*
         do i = 1, 40000
           if (vnhits(i) .gt. 0.) then
             nlines = nlines + 1

	     if (vnhits(i).eq.1) tempo = vxhits(i)
	     if (vnhits(i).gt.1) tempo = vthits(i)

             write(50,110)
     &            nlines,
     &            g_om_id_ant_om(om_num),
     &            vnhits(i),
     &            tempo,
     &         -1*p_part_id(part_num),
     &            p_iparent(part_num),
     &            part_num
           endif
         enddo
       endif
*
c 110  FORMAT('hit:',1X,I7,1X,I5,1X,F6.1,1X,F10.2,3(1X,I6))
 110  FORMAT('hit:',1X,I7,1X,I7,1X,F6.1,1X,F10.2,3(1X,I6))
 100  FORMAT(5X,'* WARNING !! PHOT_SCATT_HITS routine, gen_nhits 
     &      out of range')
*
 777   CONTINUE
*
**
*
 999  CONTINUE
      END
