#ifndef GEOMETRY
#define GEOMETRY

#include <fstream>
#include <iostream>
#include <math.h>

#ifdef ROOTSUPPORT
#include "TObject.h"
#endif
/* -------------------------------------------------------------------------- */
class Vec2D
{
  public:
    double    x, y;
  
  public:
    Vec2D(): x(0), y(0) {}  ///< Constructor
    Vec2D(double xx, double yy): x(xx), y(yy) {}  ///< Complete constructor  
    ~Vec2D() {}   ///< destructor

   inline double dot(Vec2D& v) const {return (x*v.x + y*v.y);} ///< dot product

    //! Euclidian distance- bug corrected on sept. 2001 - Th.S.  
    inline double distance(Vec2D& v) 
    const {return sqrt( (x-v.x)*(x-v.x) + (y-v.y)*(y-v.y));} 
};
/*!
\class Vec2D
\brief A much too simple class to handle 2D vectors

 Th.Stolarczyk - Bug corrected in "distance" method of Vec2D, 
 thanks to Yuri Andreev (Saclay) - sept. 2001. Previously       P. Payre, CPPM Luminy
*/

/* -------------------------------------------------------------------------- */
class Vec3D
#ifdef ROOTSUPPORT
 : public TObject
#endif
{
public:
  
  double x,y, z; // Vector components
 
public:
  
  Vec3D(): x(0), y(0), z(0) {}     ///< Default constructor: initialises with (0,0,0)
  Vec3D(double xx, double yy, double zz); ///<: x(xx), y(yy), z(zz) {}
  ~Vec3D() {}
   
  void set( double xx, double yy, double zz );///< desc: give vector value (xx,yy,zz)
  
  double dot(const Vec3D& v) const;           ///< dot-product with another vector 
  double distance(const Vec3D& v) const;      ///< distance to another vector: length(this-v)
  double length() const;                      ///< lenght of the vector
  
  Vec3D& normalize();                         ///< scale vector so its length is 1
  Vec3D& operator*=(double a);                ///< multiply with a scalar
  Vec3D& operator*=(const Vec3D& v);          ///< Multiply elementwise by another vector
  Vec3D& operator/=(double a);                ///< divide by a scalar
  Vec3D& operator+=(const Vec3D& v);          ///< add another vector to this one
  Vec3D& operator-=(const Vec3D& v);          ///< subtract another vector from this one
  std::istream& Streamer( std::istream& in );           ///< read vector from stream 
  std::ostream& Streamer( std::ostream& out ) const;    ///< write vector to stream

  #ifdef ROOTSUPPORT
  ClassDef(Vec3D,1)
  #endif
};

/* binary functions with Vec3D's */

inline Vec3D operator*(const Vec3D& v, double a) {Vec3D w=v; return w*=a; }
inline Vec3D operator/(const Vec3D& v, double a) {Vec3D w=v; return w/=a; }
inline Vec3D operator*(double a, const Vec3D& v) {Vec3D w=v; return w*=a; }
inline Vec3D operator+(const Vec3D& v, const Vec3D& w) {Vec3D q=v; return q+=w;}
inline Vec3D operator-(const Vec3D& v, const Vec3D& w) {Vec3D q=v; return q-=w;}
inline Vec3D operator*(const Vec3D& v, const Vec3D& w) {Vec3D q=v; return q*=w;}
inline Vec3D crossproduct ( const Vec3D& a, const Vec3D& b)
{
  return Vec3D ( a.y*b.z - a.z*b.y,
		 a.z*b.x - a.x*b.z,
		 a.x*b.y - a.y*b.x );
}  

/* inline function definitions for Vec3D */
inline double Vec3D::dot(const Vec3D& v) const      { return (x*v.x + y*v.y + z*v.z);}  
inline double Vec3D::distance(const Vec3D& v) const { Vec3D w=v-*this; return w.length(); }
inline double Vec3D::length() const                 
{ 
  double ll=x*x+y*y+z*z; 
  if (ll<=0) ll=0;
  return sqrt(ll);
}

inline std::ostream& operator<<(std::ostream& out ,const Vec3D& v) { return v.Streamer(out); } 
inline std::istream& operator>>(std::istream& in,        Vec3D& v) { return v.Streamer(in);  }


inline void   Vec3D::set( double xx, double yy, double zz ) { x=xx;y=yy;z=zz;}

inline Vec3D& Vec3D::operator*=(double a)             { x*=a; y*=a; z*=a; return *this; }
inline Vec3D& Vec3D::operator*=(const Vec3D& v)       { x*=v.x; y*=v.y; z*=v.z; return *this; }
inline Vec3D& Vec3D::operator/=(double a)             { x/=a; y/=a; z/=a; return *this; }
inline Vec3D& Vec3D::operator+=(const Vec3D& v)       { x+=v.x; y+=v.y; z+=v.z; return *this; }
inline Vec3D& Vec3D::operator-=(const Vec3D& v)       { x-=v.x; y-=v.y; z-=v.z; return *this; }
inline std::istream& Vec3D::Streamer( std::istream& in )        { return in >> x >> y >> z; }
inline std::ostream& Vec3D::Streamer( std::ostream& out ) const { return out << x<<" "<< y<<" "<< z; }
inline Vec3D& Vec3D::normalize()  
{ 
  double n = length();
  x/=n;y/=n;z/=n;
  return *this;
} 
/*!
\class Vec3D
\brief The class Vec3D does most of the things one may expect from a three 
dimensional vector.

A class to handle 3D vectors written by P. Payre, CPPM Luminy,
revised by Aart Heijboer (aart.heijboer@nikhef.nl).
*/
class Point;
/* -------------------------------------------------------------------------- */
class EulerAngles
{
   private:
     bool       Eu_valid;        ///< 1 if defined     
     bool       Eu_err_defined;  ///< 1 if defined   
     double     Eu_[3];          ///< radians
     double     Eu_err[3];       ///< or most general covariance matrix?
      
   public:
   
   EulerAngles();///< constructor
   EulerAngles(const double phi, const double theta, const double psi);  ///< constructor knowing angles - PP CHECK THE ORDER of phi, theta, psi ! ? ? !!
   EulerAngles(const double phi,   const double theta,   const double psi,
               const double e_phi, const double e_theta, const double e_psi);  ///< constructor knowing angles and estimated errors   
   EulerAngles(const EulerAngles&); ///< copy constructor
   ~EulerAngles() {}   ///< destructor   

   
   //! set Euler angles 
   void set(const double phi, const double theta, const double psi);

   //! set Euler angles and estimated errors
   void set(const double phi, const double theta, const double psi,
            const double e_phi, const double e_theta, const double e_psi);

   //! set Euler angles by copy of another EulerAngles              
   inline void set(const EulerAngles& angles)
   {
     Eu_valid= angles.Eu_valid;
     Eu_err_defined= angles.Eu_err_defined;
     Eu_[0]= angles.Eu_[0]; 
     Eu_[1]= angles.Eu_[1]; 
     Eu_[2]= angles.Eu_[2];
     Eu_err[0]= angles.Eu_err[0]; 
     Eu_err[1]= angles.Eu_err[1]; 
     Eu_err[2]= angles.Eu_err[2];
   }
   
   public:
   
   //============
   // GROUP I/O
   //============
   friend std::ostream& operator<<(std::ostream&, const EulerAngles&); ///< write the Point on a stream 
   friend std::istream& operator>>(std::istream&, EulerAngles&);   ///< read the event from a stream 

   //====================
   // GROUP access data
   //====================
   inline bool   valid(void)     const {return Eu_valid;} ///< check if angles are defined
   inline double phi(void)       const {return Eu_[0];}   ///< get the precession angle   
   inline double theta(void)     const {return Eu_[1];}   ///< get the nutation angle   
   inline double psi(void)       const {return Eu_[2];}   ///< get the proper rotation angle   
   inline double err_phi(void)   const {return Eu_err[0];}   ///< get the error on precession angle
   inline double err_theta(void) const {return Eu_err[1];}   ///< get the error on nutation angle   
   inline double err_psi(void)   const {return Eu_err[2];}   ///< get the error on proper rotation angle     

   //================================================================
   // GROUP rotations
   // GROUP given a reference frame "the lab" defined by
   // GROUP its origin {0,0,0} and angles {0,0,0}
   // GROUP given another frame "the solid" defined by its origin p0
   // GROUP and Euler angles (psi, theta, phi) IN THE LAB
   //================================================================

   //! if p is a Point expressed in the frame of the solid its position in the lab is:   
   Point to_lab(const Point& p0, const Point& dp) const;   

   //! if v is a vector expressed in the frame of the solid  its expression in the lab is:   
   Vec3D to_lab(const Vec3D& v) const ; 

   //! if p is a Point expressed in the lab its position in the frame of the solid is:   
   Point to_solid(const Point& p0, const Point& p) const;   

   //! if v is a vector expressed in the lab its expression in the frame of the solid is:
   Vec3D to_solid(const Vec3D& v) const;   
   
};
/*!
\class EulerAngles
\brief      A class to handle 3D rotations.

      A class to handle 3D rotations.
      here psi is precession angle, theta nutation and phi proper rotation.
      <B>PP IS IT CORRECT ? CHECK</B> -
      BEWARE: rotation of positive angle are anticlockwise!
      <BR>
      Originally written by P. Payre, CPPM Luminy
*/
/* -------------------------------------------------------------------------- */

class Point
{
   // a Point is defined in a frame (relative to an origin)
   private:
   bool       pt_valid;        ///< 1 if Point is defined  
   bool       pt_cov_defined;  ///< 1 if covariance matrix is defined  
   double     pt_pos[3];       ///< 
   double     pt_cov[3][3];    ///< most general error matrix (unfolded)

   public:
   
   //! constructor
   Point(): pt_valid(false), pt_cov_defined(false) 
   {
      pt_pos[0]=0;
      pt_pos[1]=0;
      pt_pos[2]=0;
   }
   
   Point(const double x, const double y,  const double z); ///< constructor knowing the individual coordinates
   Point(const double x,  const double y,  const double z, 
         const double ex, const double ey, const double ez); ///< constructor knowing the individual coordinates and errors    
   Point(const Point&);						///< copy constructor
   Point(const double xyz[3]);					///< constructor knowing a vector of coordinates
   Point(const double xyz[3], const double error[3]); 		///< constructor knowing a vector coordinates and a vector of errors    
   Point(const double xyz[3], const double covariance[3][3]);  	///< constructor knowing a vector coordinates and a matrix of errors   
   ~Point() {}  						///< destructor
   Point& operator=(const Point&); 				///< assignment operator
   
   //======================
   // GROUP set parameters
   //======================
   
   void set(const double x,  const double y,  const double z);  ///< set x, y, z coordinates  
   void set(const double x,  const double y,  const double z,   ///< set x, y, z coordinates and errors     
            const double ex, const double ey, const double ez);
   void set(const double xyz[3]);  				///< set coordinates from a vector 
   void set(const double xyz[3], const double error[3]);  	 ///< set coordinates and errors from two vectors     
   void set(const double xyz[3], const double covariance[3][3]);   ///< set coordinates and covariance    

   
   //!exceptional use, does not change eror matrix
   //! PP_reviewer: see this again later
   inline void set_z(double z) {pt_pos[2]=z;};
   
   //============
   // GROUP I/O
   //============
   friend std::ostream& operator<<(std::ostream&, const Point&);   ///< write on a stream 
   friend std::istream& operator>>(std::istream&, Point&);    ///< read from a stream 
                                        
   //===================
   // GROUP access data
   //===================
   
   inline bool    ok(void) const {return pt_valid;}	///< check if Point is defined   
   inline double  x(void)  const {return pt_pos[0];} 	///< get x coordinate
   inline double  y(void)  const {return pt_pos[1];} 	///< get y coordinate
   inline double  z(void)  const {return pt_pos[2];} 	///< get z coordinate
   inline const double* position(void) {return pt_pos;} ///< get coordinate as a reference to a vector  
   inline double  ex(void) const {return sqrt(pt_cov[0][0]);};  ///< get x error
   inline double  ey(void) const {return sqrt(pt_cov[1][1]);};  ///< get y error
   inline double  ez(void) const {return sqrt(pt_cov[2][2]);};  ///< get z error


};
/*!
\class Point
\brief     A class to handle 3D Points and error matrix.
      <p>A Point is defined in a frame. Written by P. Payre
*/

/* -------------------------------------------------------------------------- */

class Direction
{
private:

   bool       dv_valid;          //<! 1 if defined     
   //bool       dv_cov_defined;  //<! 1 if defined  
   double     dv_val[2];         //<! azimut, zenith in radians
   //double     dv_cov[2][2];    //<! rd^2
      
public:
   Direction(): dv_valid(false) {}   ///< constructor
   Direction(const double zenith, const double azimut);   ///< constructor from zenith and azimut   
   ////Direction(const double zenith, const double azimut, const double err_zenith, const double err_azimut);
   Direction(const double cx, const double cy, const double cz);   ///< constructor from direction cosines   
   ~Direction() {} ///< destructor
   
   //============
   // GROUP I/O
   //============
   friend std::ostream& operator<<(std::ostream&, const Direction&);///< write on a stream 
   friend std::istream& operator>>(std::istream&, Direction&);  ///< read from a stream 

   //================
   // GROUP set data
   //================
   void fill(const double zenith, const double azimut); ///< fill from zenith and azimut
   void fill(const double cx, const double cy, const double cz);   ///< fill from direction cosines
   
   //===================
   // GROUP access data
   //===================
   
   inline bool   valid(void)  const {return dv_valid;}///< check if Direction is defined      
   inline double azimut(void) const {return dv_val[0];}  ///< get azimut in radians (0= ???NORTH??? )   
   inline double zenith(void) const {return dv_val[1];}  ///< get zenith in radians  (0= upward   
   inline double cx(void) const {return sin(dv_val[1])*cos(dv_val[0]);} ///< get x direction cosine (North) 
   inline double cy(void) const {return sin(dv_val[1])*sin(dv_val[0]);}  ///< get y direction cosine (West) 
   inline double cz(void) const {return cos(dv_val[1]);}   ///< get z direction cosine (Vertical) 

};
/*!
\class Direction 
\brief  A much too simple class to handle 2D direction (like zenith, azimut) (written by P.Payre)
*/
#endif
