//===========================================================================//
// 4/3/97 P. Payre definition for 3D objects                                 //
//===========================================================================//

#include "Geometry.hh"
using namespace std;

#ifdef ROOTSUPPORT
  ClassImp(Vec3D)
#endif


//===========================================================================//
// class EulerAngles							     //
//===========================================================================//
EulerAngles::EulerAngles()
{
  Eu_valid= false;
  Eu_err_defined= false;
  Eu_[0]= 0; Eu_[1]= 0; Eu_[2]= 0; 
}

//===========================================================================//
EulerAngles::EulerAngles(const EulerAngles& other)
{
  Eu_valid= other.Eu_valid;
  Eu_err_defined= other.Eu_err_defined;
  for(int i=0;i<3;i++)
  {
    Eu_[i]= other.Eu_[i];
    Eu_err[i]= other.Eu_[i];
  }
}

//===========================================================================//
EulerAngles::EulerAngles(const double phi, const double theta, 
                           const double psi)
{
  Eu_valid= true;
  Eu_err_defined= false;
  Eu_[0]= phi; Eu_[1]= theta; Eu_[2]= psi; 
}
                           
//===========================================================================//
EulerAngles::EulerAngles(const double phi, const double theta, 
                           const double psi, const double e_phi, 
                           const double e_theta, const double e_psi)
{
  Eu_valid= true;
  Eu_err_defined= true;
  Eu_[0]= phi; Eu_[1]= theta; Eu_[2]= psi; 
  Eu_err[0]= e_phi; Eu_err[1]= e_theta; Eu_err[2]= e_psi; 
}

//===========================================================================//
void EulerAngles::set(const double phi, const double theta, const double psi)
{
  Eu_valid= true;
  Eu_err_defined= false;
  Eu_[0]= phi; Eu_[1]= theta; Eu_[2]= psi; 
}
                           
//===========================================================================//
void EulerAngles::set(const double phi, const double theta, const double psi, 
                  const double e_phi, const double e_theta, const double e_psi)
{
  Eu_valid= true;
  Eu_err_defined= true;
  Eu_[0]= phi; Eu_[1]= theta; Eu_[2]= psi; 
  Eu_err[0]= e_phi; Eu_err[1]= e_theta; Eu_err[2]= e_psi; 
}

//===========================================================================//
// read the angles from a stream 
istream& operator>>(istream& in, EulerAngles& angles)
{
  angles.Eu_valid= true;
  in >> angles.Eu_[0] >> angles.Eu_[1] >> angles.Eu_[2];
  
  return in;
}
 
//===========================================================================//
// write the angles to a stream
ostream& operator<<(ostream& out, const EulerAngles& angles)
{
  
  
  out << angles.Eu_[0] << " " << angles.Eu_[1] << " " <<  angles.Eu_[2];

  return out;
}

//===========================================================================//
// rotation of a solid around an origin 
Point EulerAngles::to_lab(const Point& p0, const Point& dp) const
{
  // model 3-D solid rotation:
  // compute the Point {x,y,z} obtained by displacing the Point p0={x0,y0,z0} 
  // along the vector dp={dx,dy,dz} rotated by{phi,theta,psi}
  // psi is precession angle, theta nutation and phi proper rotation

  const double phi=   Eu_[0];
  const double theta= Eu_[1];
  const double psi=   Eu_[2];
  
  // PP: checked with Mathematica as
  // PP: remind that mathematica has positive anangles clockwise 
  // {x,y,z}= {x0,y0,z0}+RotationMatrix3D[-phi,-theta,-psi].{dx,dy,dz}
  
  // PP_reviewer: COVARIANCE MATRIX ALSO SHOULD BE ROTATED!
  // LoOK OUT FOR ERROR PROPAGATION

  double x= p0.x()+
            dp.x()*(cos(phi)*cos(psi) - cos(theta)*sin(phi)*sin(psi)) + 
            dp.y()*(-cos(psi)*sin(phi) - cos(phi)*cos(theta)*sin(psi)) + 
            dp.z()*sin(psi)*sin(theta);
  double y= p0.y()+
            dp.x()*((cos(psi)*cos(theta)*sin(phi)) + cos(phi)*sin(psi)) + 
            dp.y()*(cos(phi)*cos(psi)*cos(theta) - sin(phi)*sin(psi)) - 
            dp.z()*cos(psi)*sin(theta);
  double z= p0.z() +
            dp.x()*sin(phi)*sin(theta) +
            dp.y()*cos(phi)*sin(theta) +
            dp.z()*cos(theta);
            
  return Point(x,y,z);
}

//===========================================================================//
Vec3D EulerAngles::to_lab(const Vec3D& v) const
{
  Vec3D u;
  const double phi=   Eu_[0];
  const double theta= Eu_[1];
  const double psi=   Eu_[2];
  
  // PP: checked with Mathematica as
  // u= RotationMatrix3D[-phi, -theta, -psi].{vx,vy,vz}
  // check OK with "sciences de l'ingenieur vol. AF4"
  u.x= v.x*(cos(phi)*cos(psi) - cos(theta)*sin(phi)*sin(psi)) + 
       v.y*(-cos(psi)*sin(phi) - cos(phi)*cos(theta)*sin(psi)) + 
       v.z*sin(psi)*sin(theta);
       
  u.y= v.x*((cos(psi)*cos(theta)*sin(phi)) + cos(phi)*sin(psi)) + 
       v.y*(cos(phi)*cos(psi)*cos(theta) - sin(phi)*sin(psi)) - 
       v.z*cos(psi)*sin(theta);
       
  u.z= v.x*sin(phi)*sin(theta) +
       v.y*cos(phi)*sin(theta) +
       v.z*cos(theta);
       
  return u;
}
   

//===========================================================================//
// rotate a lab Point into the solid, expressed in lab frame
Point EulerAngles::to_solid(const Point& p0, const Point& p) const
{
  // model 3-D solid rotation:
  // express the Point p defined by {x,y,z} in the lab in the reference frame
  // of the solid defined by the reference Point p0={x0,y0,z0}
  // and the Euler angles (phi, theta, psi) in the lab
  // NB to_solid(p0, to_lab(p0,dp)) === dp
  // NB to_lab(p0, to_solid(p0,p)) === p
  
  const double phi=   Eu_[0];
  const double theta= Eu_[1];
  const double psi=   Eu_[2];
  
  // PP: checked with Mathematica as
  // {dx,dy,dz}= RotationMatrix3D[psi,theta,phi].{x-x0,y-y0,z-z0}

  // PP_reviewer: COVARIANCE MATRIX ALSO SHOULD BE ROTATED!
  // LoOK OUT FOR ERROR PROPAGATION
  
  double dx=(p.x()-p0.x())*(cos(phi)*cos(psi)-cos(theta)*sin(phi)*sin(psi))+
            (p.y()-p0.y())*(cos(psi)*cos(theta)*sin(phi)+cos(phi)*sin(psi))+
            (p.z()-p0.z())*sin(phi)*sin(theta);
  
  double dy=(p.x()-p0.x())*(-cos(psi)*sin(phi)-cos(phi)*cos(theta)*sin(psi))+
            (p.y()-p0.y())*(cos(phi)*cos(psi)*cos(theta)-sin(phi)*sin(psi))+
            (p.z()-p0.z())*cos(phi)*sin(theta);
  
  double dz=(p.x()-p0.x())*sin(psi)*sin(theta)-
            (p.y()-p0.y())*cos(psi)*sin(theta)+
            (p.z()-p0.z())*cos(theta);
              
  return Point(dx,dy,dz);
}

//===========================================================================//
Vec3D EulerAngles::to_solid(const Vec3D& v) const
{
  Vec3D u;
  const double phi=   Eu_[0];
  const double theta= Eu_[1];
  const double psi=   Eu_[2];
    
  // PP: checked with Mathematica as
  // u= RotationMatrix3D[psi, theta, phi].{vx,vy,vz}
  u.x= v.x*(cos(phi)*cos(psi) - cos(theta)*sin(phi)*sin(psi)) + 
       v.y*(cos(phi)*sin(psi) + cos(psi)*cos(theta)*sin(phi)) + 
       v.z*sin(phi)*sin(theta);
       
  u.y= v.x*(-cos(psi)*sin(phi) -cos(phi)*cos(theta)*sin(psi)) + 
       v.y*(cos(phi)*cos(psi)*cos(theta) - sin(phi)*sin(psi)) + 
       v.z*cos(phi)*sin(theta);
       
  u.z= v.x*sin(psi)*sin(theta) -
       v.y*cos(psi)*sin(theta) +
       v.z*cos(theta);
              
  return u;
}   

//===========================================================================//
// class Point 								     //
//===========================================================================//
Point::Point(const Point& pt)
{
  pt_valid= pt.pt_valid;
  pt_cov_defined= pt.pt_cov_defined;
  
  if(pt_valid) for(int i=0; i< 3; i++) pt_pos[i]= pt.pt_pos[i];
  
  if(pt_cov_defined)
  {
    for(int i=0; i< 3; i++)
    { 
      for(int j=0; j< 3; j++) pt_cov[i][j]= pt.pt_cov[i][j];
    }
  } 
}


//===========================================================================//
Point::Point(const double x, const double y, const double z)
{ 
  set(x,y,z);   
}

//===========================================================================//
Point::Point(const double x, const double y, const double z,
             const double ex, const double ey, const double ez)
{ 
  set(x,y,z,ex,ey,ez);   
}

//===========================================================================//
Point::Point(const double xyz[3])
{    
  set(xyz);   
}

//===========================================================================//
Point::Point(const double xyz[3], const double error[3])
{    
  set(xyz,error);   
}

//===========================================================================//
Point::Point(const double xyz[3], const double covariance[3][3])
{    
  set(xyz, covariance);   
}

//===========================================================================//
// =
Point& Point::operator=(const Point& other)
{
  pt_valid= other.pt_valid;
  pt_cov_defined= other.pt_cov_defined;
  for(int i=0; i< 3; i++) pt_pos[i]= other.pt_pos[i];
  for(int i=0; i< 3; i++)
  { 
    for(int j=0; j< 3; j++) pt_cov[i][j]= other.pt_cov[i][j];
    
  } 
  return *this;
}

//===========================================================================//
// read the Point from a stream 
istream& operator>>(istream& in, Point& pt)
{

  pt.pt_valid=true;

  in >> pt.pt_pos[0] >> pt.pt_pos[1] >> pt.pt_pos[2] ;
    
  return in;
}

//===========================================================================//
// write the Point to a stream 
ostream& operator<<(ostream& out, const Point& pt)
{

  // else write position
  out  << " " << pt.pt_pos[0] << " " << pt.pt_pos[1] << " " << pt.pt_pos[2] ;

  return out;
}

//===========================================================================//
// set values
//===========================================================================//
void Point::set(const double x, const double y, const double z)
{    
  // define the state
  pt_valid= true; 
  pt_cov_defined= false;
  
  pt_pos[0]= x; pt_pos[1]= y; pt_pos[2]= z;  
}

//===========================================================================//
void Point::set(const double x, const double y, const double z,
             const double ex, const double ey, const double ez)
{    
  // define the state
  pt_valid= true; 
  pt_cov_defined= true;
  
  pt_pos[0]= x;  pt_pos[1]= y;  pt_pos[2]= z;
  
  for(int i=0; i<3; i++) for(int j=0; j<3; j++) pt_cov[i][j]=0; 
  pt_cov[0][0]= ex*ex; pt_cov[1][1]= ey*ey; pt_cov[2][2]= ez*ez;
}

//===========================================================================//
void Point::set(const double xyz[3])
{    
  // define the state
  pt_valid= true; 
  pt_cov_defined= false;

  // copy...
  for(int i=0; i<3; i++) pt_pos[i]= xyz[i];  
}

//===========================================================================//
void Point::set(const double xyz[3], const double error[3])
{    
  // define the state
  pt_valid= true; 
  pt_cov_defined= true;

  // copy...
  for(int i=0; i<3; i++) for(int j=0; j<3; j++) pt_cov[i][j]=0; 
  for(int i=0; i<3; i++) 
  {
    pt_pos[i]= xyz[i];
    pt_cov[i][i]= error[i]*error[i];
  }
}

//===========================================================================//
void Point::set(const double xyz[3], const double covariance[3][3])
{    
  // define the state
  pt_valid= true; 
  pt_cov_defined= true;

  // copy...
  for(int i=0; i<3; i++) pt_pos[i]= xyz[i];
  for(int i=0; i<3; i++) for(int j=0; j<3; j++) pt_cov[i][j]= covariance[i][j]; 
}

//===========================================================================//
// class Direction							     //
//===========================================================================//

//===========================================================================//
Direction::Direction(const double zenith, const double azimut)
{    
  dv_valid= true; 
  //dv_cov_defined= false;
    
  dv_val[0]= azimut;
  dv_val[1]= zenith;
}

//===========================================================================//
Direction::Direction(const double cx, const double cy, const double cz)
{    
  dv_valid= true; 
  //dv_cov_defined= false;
  
  double r= sqrt(cx*cx + cy*cy + cz*cz);  
  dv_val[1]= acos(cz/r);
  
  //use continuity to avoid nasty 
  if(cx!=0) dv_val[0]= atan2(cy,cx);
  else      dv_val[0]= atan2(cy,1e-18);
}

//===========================================================================//
void Direction::fill(const double zenith, const double azimut)
{    
  dv_valid= true; 
  //dv_cov_defined= false;
    
  dv_val[0]= azimut;
  dv_val[1]= zenith;
}

//===========================================================================//
void Direction::fill(const double cx, const double cy, const double cz)
{    
  dv_valid= true; 
  //dv_cov_defined= false;
  
  double r= sqrt(cx*cx + cy*cy + cz*cz);  
  dv_val[1]= acos(cz/r);
  
  //use continuity to avoid nasty 
  if(cx!=0) dv_val[0]= atan2(cy,cx);
  else      dv_val[0]= atan2(cy,1e-18);
}

//===========================================================================//
// read the Direction from a stream 
istream& operator>>(istream& in, Direction& d)
{
  in >> d.dv_valid ;
  // >> d.err_defined >> d.cov_defined;

  // if not valid nothing else to read
  if(! d.dv_valid) return in;
  
  in >> d.dv_val[0] >> d.dv_val[1] ;
  

  return in;
}

//===========================================================================//
// write the Point to a stream 
ostream& operator<<(ostream& out, const Direction& d)
{
  // first flags
  out << d.dv_valid << " " ;
  ////  << d.err_defined << " " << d.cov_defined ;

  // if not valid nothing else to write
  if(! d.valid() ) return out;
  
  // else write position
  out << " " << d.dv_val[0] << " " << d.dv_val[1] ;

  return out;
}

Vec3D::Vec3D(double xx, double yy, double zz): x(xx), y(yy), z(zz) {} 
