// vec3d.h   -*-C++-*-

// vec3d class
//
// based on GGVector from Graphics Gems
//

#ifndef _VEC3D_H
#define _VEC3D_H 1

#include <iostream.h>
#include <math.h>
#include <stdlib.h>

#include "Float.h"

class vec3d {
protected:
  static Float epsilon;
  // use a small array for easy conversion for SGI gl lib v3d function
  Float d[3];
public:
  // constructors/destructor
  vec3d();
  vec3d(const Float x, const Float y, const Float z);
  vec3d(const vec3d& v);
  // access routines
  Float x() const; // get component
  Float y() const;
  Float z() const;

  Float& operator [] (int);
  const Float& operator [] (int) const;

  void sx(const Float x); // set component
  void sy(const Float y);
  void sz(const Float z);

  static Float get_epsilon();
  static void set_epsilon(const Float);

  const Float* v3d() const; // get d for use in v3d() (const may have to go) 


  // operators
  vec3d& operator += (const vec3d& v); // t = t + v
  vec3d& operator -= (const vec3d& v); // t = t - v

  vec3d& operator *= (const Float v); // mult each component by v
  vec3d& operator /= (const Float v); // div each component by v

  // operators on vector
  Float dot(const vec3d&) const; // vector dot product
  Float normalize(); // normalize vector and return old length
  Float len() const; // return length
  Float sqlen() const; // return square of length

  // friend functions
  friend int operator == (const vec3d& a, const vec3d& b); // |a-b| < epsilon
  friend int operator != (const vec3d& a, const vec3d& b); // |a-b| >= epsilon
  friend int operator < (const vec3d& a, const vec3d& b); // |a| < |b|

  friend vec3d operator - (const vec3d& a); // negation of a
  friend vec3d operator + (const vec3d& a, const vec3d& b); // vector addition
  friend vec3d operator - (const vec3d& a, const vec3d& b);            
  friend vec3d operator * (const Float a, const vec3d& b); // mult or div each
  friend vec3d operator * (const vec3d& a, const Float b); // component
  friend vec3d operator / (const vec3d& a, const Float b); // by the Float

  friend Float dot(const vec3d& a, const vec3d& b); // vector dot product
  friend vec3d cross(const vec3d& a, const vec3d& b);// vector cross product

  friend Float sqlen(const vec3d& v);     // square of the length of v
  friend Float len(const vec3d& v);       // length of v
  friend vec3d normalize(const vec3d& v); // normalized vector v

  // scales v to the new length 
  friend vec3d scale(const vec3d& v, const Float newlen); 

  // linearly interpolate between vectors by alpha,
  // if alpha = 0, result = lo, if alpha = 1 result = hi
  friend vec3d interpolate(const vec3d& hi, const vec3d& lo, 
				  const Float alpha);

  // linear combination of two vectors            
  // result = ( a * as) + ( b * bs )
  friend vec3d combine(const vec3d& a, const Float as, 
		       const vec3d &b, const Float bs);

  // distance between a & b
  friend Float distance(const vec3d& a, const vec3d& b);

};

// further function declartions

istream& operator >> (istream& s, vec3d& v); // get from istream
ostream& operator << (ostream& s, const vec3d& v); // print on ostream

// inline members

inline Float vec3d::x() const { return d[0]; }
inline Float vec3d::y() const { return d[1]; }
inline Float vec3d::z() const { return d[2]; }

inline void vec3d::sx(const Float x) { d[0] = x; }
inline void vec3d::sy(const Float y) { d[1] = y; }
inline void vec3d::sz(const Float z) { d[2] = z; }

inline Float& vec3d::operator [] (int i)
{
  if(i<3 && i>=0) return d[i];
  cerr << "vec3d: bad subscript " << i << endl;
  exit(1);
  return d[0];
}

inline const Float& vec3d::operator [] (int i) const
{
  if(i<3 && i>=0) return d[i];
  cerr << "vec3d: bad subscript " << i << endl;
  exit(1);
  return d[0];
}

inline const Float* vec3d::v3d() const { return d; }

// constructor/destructor
inline vec3d::vec3d() { d[0] = 0; d[1] = 0; d[2] = 0; }

inline vec3d::vec3d(const Float x, const Float y, const Float z) 
{ 
  d[0] = x; d[1] = y; d[2] = z;
}

inline vec3d::vec3d(const vec3d& v)
{
  d[0] = v.d[0]; d[1] = v.d[1]; d[2] = v.d[2];
}

// operators/friend functions

inline vec3d& vec3d::operator += (const vec3d& v) 
{
  d[0] += v.d[0]; d[1] += v.d[1]; d[2] += v.d[2];
  return *this;
}

inline vec3d& vec3d::operator -= (const vec3d& v) 
{
  d[0] -= v.d[0]; d[1] -= v.d[1]; d[2] -= v.d[2];
  return *this;
}

inline vec3d& vec3d::operator *= (const Float v)
{
  d[0] *= v; d[1] *= v; d[2] *= v;
  return *this;
}

inline vec3d& vec3d::operator /= (const Float v)
{
  d[0] /= v; d[1] /= v; d[2] /= v;
  return *this;
}

inline vec3d operator - (const vec3d& v)
{
  return vec3d(-v.d[0], -v.d[1], -v.d[2]);
}

inline vec3d operator + (const vec3d& a, const vec3d& b)
{
  return vec3d(a.d[0] + b.d[0], a.d[1] + b.d[1], a.d[2] + b.d[2]);
}

inline vec3d operator - (const vec3d& a, const vec3d& b)
{
  return vec3d(a.d[0] - b.d[0], a.d[1] - b.d[1], a.d[2] - b.d[2]);
}

inline vec3d operator * (const Float a, const vec3d& b)
{
  return vec3d(a * b.d[0], a * b.d[1], a * b.d[2]);
}

inline vec3d operator * (const vec3d& a, const Float b)
{
  return vec3d(a.d[0] * b, a.d[1] * b, a.d[2] * b);
}

inline vec3d operator / (const vec3d& a, const Float b)
{
  return vec3d(a.d[0] / b, a.d[1] / b, a.d[2] / b);
}

inline Float sqlen(const vec3d& v)
{
  return v.d[0]*v.d[0] + v.d[1] * v.d[1] + v.d[2] * v.d[2];
}

inline int operator == (const vec3d& a, const vec3d &b) 
{
  return sqlen(a-b) < vec3d::epsilon;
}

inline int operator != (const vec3d& a, const vec3d &b)
{
  return sqlen(a-b) >= vec3d::epsilon;
}

inline int operator < (const vec3d& a, const vec3d &b) {
  return sqlen(a) < sqlen(b);
}

inline Float vec3d::dot(const vec3d &b) const
{
  return (d[0]*b.d[0] + d[1]*b.d[1] + d[2]*b.d[2]);
}

inline Float 
vec3d::sqlen() const
{
  return d[0]*d[0] + d[1] * d[1] + d[2] * d[2];
}

inline Float vec3d::len() const
{
  return sqrt(sqlen());
}

inline Float 
vec3d::normalize()
{
  Float l = sqlen();
  if(l == 1.0) return l;
  if(l == 0.0) return 0;
  l = sqrt(l);
  d[0] /= l; d[1] /= l; d[2] /= l;
  return l;
}

inline Float dot(const vec3d& a, const vec3d& b)
{
  return (a.d[0] * b.d[0] + a.d[1] * b.d[1] + a.d[2] * b.d[2]);
}

inline vec3d cross(const vec3d& a, const vec3d& b)
{
  return vec3d((a.d[1]*b.d[2]) - (a.d[2]*b.d[1]),
	       (a.d[2]*b.d[0]) - (a.d[0]*b.d[2]),
	       (a.d[0]*b.d[1]) - (a.d[1]*b.d[0]));
}

inline Float len(const vec3d& v)
{
  return sqrt(sqlen(v));
}

inline vec3d normalize(const vec3d& v)
{
  vec3d nv = v;
  nv.normalize();
  return nv;
}

inline vec3d scale(const vec3d& v, const Float newlen)
{
  Float l = len(v);
  
  return v * (newlen / l);
}

inline vec3d interpolate(const vec3d& hi, const vec3d& lo, 
			 const Float alpha)
{
  return vec3d(lo.d[0] + ((hi.d[0]-lo.d[0])*alpha),
	       lo.d[1] + ((hi.d[1]-lo.d[1])*alpha),
	       lo.d[2] + ((hi.d[2]-lo.d[2])*alpha));
}

inline Float vec3d::get_epsilon() {
  return vec3d::epsilon;
}

inline void vec3d::set_epsilon(const Float d) {
  vec3d::epsilon = d;
}

#endif
